# From http://www.koders.com/python/fid1C296EC5EC10F9A979E070FFD573AD7946D53274.aspx

"""Simple module providing a quaternion class for manipulating rotations easily.

Note: all angles are assumed to be specified in radians.
Note: this is an entirely separate implementation from the PyOpenGL
	quaternion class.  This implementation assumes that Numeric python
	will be available, and provides only those methods and helpers
	commonly needed for manipulating rotations.
"""
from math import *
from numpy import *
# from Numeric import *
# from OpenGLContext import utilities




class Quaternion(object):
	"""Quaternion object implementing those methods required
	to be useful for OpenGL rendering (and not many others)"""
	__slots__ = ('internal','__weakref__')
	def __init__ (self, elements = [1,0,0,0] ):
		"""The initializer is a four-element array,
		
		w, x,y,z -- all elements should be doubles/floats
		the default values are those for a unit multiplication
		quaternion.
		"""
		elements = asarray( elements, 'd')
		length = sqrt( sum( elements * elements))
		if length != 1:
##			print 'fixing quaternion length', repr(length)
			elements = elements/length
		self.internal = elements
	def __mul__( self, other ):
		"""Multiply this quaternion by another quaternion,
		generating a new quaternion which is the combination of the
		rotations represented by the two source quaternions.

		Other is interpreted as taking place within the coordinate
		space defined by this quaternion.

		Alternately, if "other" is a matrix, return the dot-product
		of that matrix with our matrix (i.e. rotate the coordinate)
		"""
		if hasattr( other, 'internal' ):
			w1,x1,y1,z1 = self.internal
			w2,x2,y2,z2 = other.internal
			
			w = w1*w2 - x1*x2 - y1*y2 - z1*z2
			x = w1*x2 + x1*w2 + y1*z2 - z1*y2
			y = w1*y2 + y1*w2 + z1*x2 - x1*z2
			z = w1*z2 + z1*w2 + x1*y2 - y1*x2
			return self.__class__( array([w,x,y,z],'d'))
		else:
			return dot( self.matrix (), other )
	def XYZR( self ):
		"""Get a VRML-style axis plus rotation form of the rotation.
		Note that this is in radians, not degrees, and that the angle
		is the last, not the first item... (x,y,z,radians)
		"""
		w,x,y,z = self.internal
		try:
			aw = acos(w)
		except ValueError:
			# catches errors where w == 1.00000000002
			aw = 0
		scale = sin(aw)
		if not scale:
			return (0,1,0,0)
		return (x / scale, y / scale, z / scale, 2 * aw )
	def matrix( self ):
		"""Get a rotation matrix representing this rotation"""
		w,x,y,z = self.internal
		return	[ 1-2*y*y-2*z*z, 2*x*y+2*w*z, 2*x*z-2*w*y, 0,
			2*x*y-2*w*z, 1-2*x*x-2*z*z, 2*y*z+2*w*x, 0,
			2*x*z+2*w*y, 2*y*z-2*w*x, 1-2*x*x-2*y*y, 0,
			0, 0, 0, 1]
#		return array([
#			[ 1-2*y*y-2*z*z, 2*x*y+2*w*z, 2*x*z-2*w*y, 0],
#			[ 2*x*y-2*w*z, 1-2*x*x-2*z*z, 2*y*z+2*w*x, 0],
#			[ 2*x*z+2*w*y, 2*y*z-2*w*x, 1-2*x*x-2*y*y, 0],
#			[ 0,0,0,1],
#		])
	def glMatrix( self ):
		"""Get a rotation matrix representing this rotation"""
		w,x,y,z = self.internal
		return	[ 1-2*y*y-2*z*z, 2*x*y-2*w*z, 2*x*z+2*w*y, 0,
			2*x*y+2*w*z, 1-2*x*x-2*z*z, 2*y*z-2*w*x, 0,
			2*x*z-2*w*y, 2*y*z+2*w*x, 1-2*x*x-2*y*y, 0,
			0, 0, 0, 1]

	def __getitem__( self, x ):
		return self.internal[x]
	def __len__( self ):
		return len( self.internal)
	def __repr__( self ):
		"""Return a human-friendly representation of the quaternion

		Currently this representation is as an axis plus rotation (in radians)
		"""
		return """<%s XYZR=%s>"""%( self.__class__.__name__, list(self.XYZR()))
	def delta( self, other ):
		"""Return the angle in radians between this quaternion and another.

		Return value is a positive angle in the range 0-pi representing
		the minimum angle between the two quaternion rotations.
		
		From code by Halldor Fannar on the 3D game development algos list
		"""
		#first get the dot-product of the two vectors
		cosValue = sum(self.internal + other.internal)
		# now get the positive angle in range 0-pi
		return acos( cosValue )
	def slerp( self, other, fraction = 0, minimalStep= 0.0001):
		"""Perform fraction of spherical linear interpolation from this quaternion to other quaternion

		Algo is from: http://www.gamasutra.com/features/19980703/quaternions_01.htm
		"""
		cosValue = sum(self.internal * other.internal)
		# if the cosValue is negative, use negative target and cos values?
		# not sure why, it's just done this way in the sample code
		if cosValue < 0.0:
			cosValue = -cosValue
			target = -other.internal
		else:
			target = other.internal[:]
		if (1.0- cosValue) > minimalStep:
			# regular spherical linear interpolation
			angle = acos( cosValue )
			angleSin = sin( angle )
			sourceScale = sin( (1.0- fraction) * angle ) / angleSin
			targetScale = sin( fraction * angle ) / angleSin
		else:
			sourceScale = 1.0-fraction
			targetScale = fraction
		return self.__class__( (sourceScale * self.internal)+(targetScale * target) )

	@staticmethod
	def fromXYZR( x,y,z, r ):
		"""Create a new quaternion from a VRML-style rotation
		x,y,z are the axis of rotation
		r is the rotation in radians."""
		
	#	x,y,z = utilities.normalise( (x,y,z) )
		# Normalize x, y, z
		norm = sqrt( sum( map( lambda(i): i*i, (x,y,z) ) ) )
		x,y,z =  (x/norm, y/norm, z/norm)
		
		return Quaternion ( array( [
			cos(r/2.0), x*(sin(r/2.0)), y*(sin(r/2.0)), z*(sin(r/2.0)),
		]) )
		
	@staticmethod
	def fromEuler( x=0,y=0,z=0 ):
		"""Create a new quaternion from a 3-element euler-angle
		rotation about x, then y, then z
		"""
		if x:
			base = Quaternion.fromXYZR( 1,0,0,x)
			if y:
				base = base * Quaternion.fromXYZR( 0,1,0,y)
			if z:
				base = base * Quaternion.fromXYZR( 0,0,1,z)
			return base
		elif y:
			base = Quaternion.fromXYZR( 0,1,0,y)
			if z:
				base = base * Quaternion.fromXYZR( 0,0,1,z)
			return base
		else:
			return Quaternion.fromXYZR( 0,0,1,z)
	
	
def test ():
	print 'fromEuler'
	print Quaternion.fromEuler( pi/2 ).XYZR()
	print Quaternion.fromEuler( y = pi/2 ).XYZR()
	print Quaternion.fromEuler( z = pi/2 ).XYZR()
	print Quaternion.fromEuler( y = pi/2, z = pi/2 ).matrix()
	rot = Quaternion.fromEuler( y = pi/2, z = pi/2 ).XYZR()
	print apply( Quaternion.fromXYZR, rot).matrix()
	print Quaternion.fromEuler( y = pi/2, z = pi/2 )
	first = Quaternion.fromXYZR( 0,1,0,0 )
	second = Quaternion.fromXYZR( 0,1,0,pi )
	for fraction in arange( 0.0, 1.0, .01 ):
		print first.slerp( second, fraction )

if __name__== "__main__":
	test ()



import wx
import wx.glcanvas 

import zgl.zgl_graphdrawer.application as ApplicationModule

class wxApplication(ApplicationModule.Application, wx.PySimpleApp):

    def __init__(self):
        wx.PySimpleApp.__init__(self)
        Application.__init__(self)
        return
        
    def OnInit(self):
        print "init now"
        return True

    # END class wxApplication
    pass



import Image
from numpy import *
from OpenGL.GL import *
from zglUtils import *
from zglDrawer import *


#==============================================================
#
#	An Object that's based on an array of vertices
#
#
class zglVertexObject(zglObject):
	def __init__(self, vertices, colors, faces, normals):
		zglObject.__init__(self)
		self.size = 1.0;
		
		self.vertices = vertices[:]
		
		if colors is not None:
			self.colors = colors[:]
		else:
			self.colors = None
			
		if faces is not None:
			self.faces = faces[:]
		else:
			self.faces = None
			
		if normals is not None:
			self.normals = normals[:]
		else:
			self.normals = None

#		self.lines = []			#
		
	
	def getVertexArray(self):
		a = []
		for face_index in range(0,len(self.faces)):
			face = self.faces[face_index]
			for vertex_index in face:
				a.extend( [self.vertices[vertex_index][0], self.vertices[vertex_index][1], self.vertices[vertex_index][2]] )
		return a
	
	def getColorArray(self):
		if self.colors is None: return None
		
		a = []
		for face_index in range(0,len(self.faces)):
			face = self.faces[face_index]
			for vertex_index in face:
				a.extend( [self.colors[vertex_index][0], self.colors[vertex_index][1], self.colors[vertex_index][2], self.alpha] )
		return a


	# Assume the that vertices never exceed -1 or 1 in any direction
	# in that case the object should be completely enclosed in a -1 to 1 box
	# which in turn is completely encased by a sphere of size sqrt(3)
	#
	def getBoundingRadius(self):
		return SQRT_3 * self.size


	def maindraw(self):
		glBegin(GL_QUADS)
		for face_index in range(0,len(self.faces)):
			face = self.faces[face_index]
			
			if self.normals is not None:
				glNormal3f(self.normals[face_index][0], self.normals[face_index][1], self.normals[face_index][2])
			
			for vertex_index in face:
				if self.colors is not None:
					glColor4f(self.colors[vertex_index][0], self.colors[vertex_index][1], self.colors[vertex_index][2], self.alpha)
					
				glVertex3f(self.vertices[vertex_index][0] * self.size, self.vertices[vertex_index][1] * self.size, self.vertices[vertex_index][2] * self.size)
				
		glEnd()
		
	
	def drawStroke(self):
		self.predraw()
		glColor4f(self.colour[0], self.colour[1], self.colour[2], self.colour[3]*self.alpha)
		for face in self.faces:
			glBegin(GL_LINE_LOOP)
			for vertex_index in face:
				glVertex3f(self.vertices[vertex_index][0] * self.size, self.vertices[vertex_index][1] * self.size, self.vertices[vertex_index][2] * self.size)
			glEnd()
		self.postdraw()




class zglVertexRect(zglVertexObject):
	vertices = ( (1.0, 1.0, 0.0), (1.0, -1.0, 1.0), (-1.0, -1.0, 1.0), (-1.0, 1.0, 1.0) )
	faces = ( (3, 2, 1, 0) )
	normals = ( (0, 0, 1) )

	

class zglCube(zglVertexObject):
	# Cube Vertices: 8 vertex, 3 coords each
	vertices = (
	(1.0, 1.0, 1.0), (1.0, -1.0, 1.0), (-1.0, -1.0, 1.0), (-1.0, 1.0, 1.0),
	(1.0, 1.0, -1.0), (1.0, -1.0, -1.0), (-1.0, -1.0, -1.0), (-1.0, 1.0, -1.0) )
	
	# Cube Colour Vertices: 8 vertex, 3 colour coords each
	colors = (
	(1.0, 1.0, 1.0), (1.0, 1.0, 0.0), (0.0, 1.0, 0.0), (0.0, 1.0, 1.0),
	(1.0, 0.0, 1.0), (1.0, 0.0, 0.0), (0.0, 0.0, 0.0), (0.0, 0.0, 1.0) )
	
	# Cube Faces: 6 faces, 4 vertex each
	faces = ( (3, 2, 1, 0), (2, 3, 7, 6), (0, 1, 5, 4), (3, 0, 4, 7), (1, 2, 6, 5), (4, 5, 6, 7) )
	
	# Cube Normals
	normals = ( (0, 0, 1), (-1, 0, 0), (1, 0, 0), (0, 1, 0), (0, -1, 0), (0, 0, -1) )
	
	def __init__(self):
		zglVertexObject.__init__(self, 	zglCube.vertices, 
										zglCube.colors, 
										zglCube.faces, 
										zglCube.normals )


#==============================================================
#
#	An vertex array object that's drawn using GL's vertex array
#
#	COMBINE WITH VERTEX OBJECT< SUBCLASS???
#
class zglVertexArray(zglObject):
	def __init__(self, vertices, colors=None, mode=GL_TRIANGLES):
		zglObject.__init__(self)
		self.vertices = vertices
		self.colors = colors
		self.mode = mode
		
		if self.colors is None:
			self.use_colour_array = False
		else:
			self.use_colour_array = True
		
		self.length = int( len(self.vertices)/3 )
		

	# MAKE SURE THE MODE IS THE SAME	
	def addVertices(self, vertices, colors=None):
		pass
	
	def addVertexObject(self, vertex_object):
		pass
	

		
	def maindraw(self):
		glEnableClientState(GL_VERTEX_ARRAY)
		
		if self.use_colour_array:
			glEnableClientState(GL_COLOR_ARRAY)
		
		self.predraw()
		
		if self.use_colour_array:
			glColorPointer(	4, 				# coordinates per vertex. We want alpha, so 4
							GL_FLOAT, 		# array data type
							0, 				# stride. 0 means no space between vertex data
							self.colors)	# The array
		glVertexPointer(3, 
						GL_FLOAT, 
						0, 
						self.vertices)
		glDrawArrays(	self.mode,		# render mode
						0, 				# Starting vertex 
						self.length)	# number of vertices to be rendered
		
		self.postdraw()
		
		glDisableClientState(GL_VERTEX_ARRAY)
		
		if self.use_colour_array:
			glDisableClientState(GL_COLOR_ARRAY)




#	A quick and simple opengl font library that uses GNU freetype2, written
#	and distributed as part of a tutorial for nehe.gamedev.net.
#	Sven Olsen, 2003
#	Translated to PyOpenGL by Brian Leair, 2004
# 
#



# import freetype
# We are going to use Python Image Library's font handling
# From PIL 1.1.4:
import ImageFont
from OpenGL.GL import *
from OpenGL.GLU import *


# Python 2.2 defines these directly
try:
	True
except NameError:
	True = 1==1
	False = 1==0


class zglFreeType:
	def __init__ (self, facename, pixel_height):
		# We haven't yet allocated textures or display lists
		self.m_allocated = False
		self.m_font_height = pixel_height
		self.m_facename = facename
		
		### since I copied this from the Nehe example, older files still use glPrint
		### but I prefer the name draw
		self.glPrint = self.draw

		# Try to obtain the FreeType font
		try:
			self.font = ImageFont.truetype(facename, pixel_height)
		except:
			raise ValueError, "Unable to locate true type font '%s'" % (facename)

		# Here we ask opengl to allocate resources for
		# all the textures and displays lists which we
		# are about to create.  
		self.m_list_base = glGenLists (128)

		# Consturct a list of 128 elements. This
		# list will be assigned the texture IDs we create for each glyph
		self.textures = [None] * 128

		# This is where we actually create each of the fonts display lists.
		for i in xrange (128):
			zglFreeType.make_dlist (self.font, i, self.m_list_base, self.textures);

		self.m_allocated = True


		# //We don't need the face information now that the display
		# //lists have been created, so we free the assosiated resources.
		# Note: Don't need this, as python will decref and dealloc the ft for us.
		return

	@staticmethod
	def is_font_available (ft, facename):
		""" Returns true if FreeType can find the requested face name 
			Pass the basname of the font e.g. "arial" or "times new roman"
		"""
		if (facename in ft.available_fonts ()):
			return True
		return False
	
	
	@staticmethod
	def next_p2 (num):
		""" If num isn't a power of 2, will return the next higher power of two """
		rval = 1
		while (rval<num):
			rval <<= 1
		return rval
	
	
	
	@staticmethod
	def make_dlist (ft, ch, list_base, tex_base_list):
		""" Given an integer char code, build a GL texture into texture_array,
			build a GL display list for display list number display_list_base + ch.
			Populate the glTexture for the integer ch and construct a display
			list that renders the texture for ch.
			Note, that display_list_base and texture_base are supposed
			to be preallocated for 128 consecutive display lists and and 
			array of textures.
		"""
	
		# //The first thing we do is get FreeType to render our character
		# //into a bitmap.  This actually requires a couple of FreeType commands:
		# //Load the Glyph for our character.
		# //Move the face's glyph into a Glyph object.
		# //Convert the glyph to a bitmap.
		# //This reference will make accessing the bitmap easier
		# - This is the 2 dimensional Numeric array
	
		# Use our helper function to get the widths of
		# the bitmap data that we will need in order to create
		# our texture.
		glyph = ft.getmask (chr (ch))
		glyph_width, glyph_height = glyph.size 
		# We are using PIL's wrapping for FreeType. As a result, we don't have 
		# direct access to glyph.advance or other attributes, so we add a 1 pixel pad.
		width = zglFreeType.next_p2 (glyph_width + 1)
		height = zglFreeType.next_p2 (glyph_height + 1)
	
	
		# python GL will accept lists of integers or strings, but not Numeric arrays
		# so, we buildup a string for our glyph's texture from the Numeric bitmap 
	
		# Here we fill in the data for the expanded bitmap.
		# Notice that we are using two channel bitmap (one for
		# luminocity and one for alpha), but we assign
		# both luminocity and alpha to the value that we
		# find in the FreeType bitmap. 
		# We use the ?: operator so that value which we use
		# will be 0 if we are in the padding zone, and whatever
		# is the the Freetype bitmap otherwise.
		expanded_data = ""
		for j in xrange (height):
			for i in xrange (width):
				if (i >= glyph_width) or (j >= glyph_height):
					value = chr (0)
					expanded_data += value
					expanded_data += value
				else:
					value = chr (glyph.getpixel ((i, j)))
					expanded_data += value
					expanded_data += value
	
		# -------------- Build the gl texture ------------
	
		# Now we just setup some texture paramaters.
		ID = glGenTextures (1)
		tex_base_list [ch] = ID
		glBindTexture (GL_TEXTURE_2D, ID)
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
	
		border = 0
		# Here we actually create the texture itself, notice
		# that we are using GL_LUMINANCE_ALPHA to indicate that
		# we are using 2 channel data.
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, width, height,
			border, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, expanded_data )
	
		# With the texture created, we don't need to expanded data anymore
		expanded_data = None
	
	
	
		# --- Build the gl display list that draws the texture for this character ---
	
		# So now we can create the display list
		glNewList (list_base + ch, GL_COMPILE)
	
		if (ch == ord (" ")):
			glyph_advance = glyph_width
			glTranslatef(glyph_advance, 0, 0)
			glEndList()
		else:
	
			glBindTexture (GL_TEXTURE_2D, ID)
	
			glPushMatrix()
	
			# // first we need to move over a little so that
			# // the character has the right amount of space
			# // between it and the one before it.
			# glyph_left = glyph.bbox [0]
			# glTranslatef(glyph_left, 0, 0)
	
			# // Now we move down a little in the case that the
			# // bitmap extends past the bottom of the line 
			# // this is only true for characters like 'g' or 'y'.
			# glyph_descent = glyph.decent
			# glTranslatef(0, glyph_descent, 0)
	
			# //Now we need to account for the fact that many of
			# //our textures are filled with empty padding space.
			# //We figure what portion of the texture is used by 
			# //the actual character and store that information in 
			# //the x and y variables, then when we draw the
			# //quad, we will only reference the parts of the texture
			# //that we contain the character itself.
			x=float (glyph_width) / float (width)
			y=float (glyph_height) / float (height)
	
			# //Here we draw the texturemaped quads.
			# //The bitmap that we got from FreeType was not 
			# //oriented quite like we would like it to be,
			# //so we need to link the texture to the quad
			# //so that the result will be properly aligned.
			glBegin(GL_QUADS)
			glTexCoord2f(0,0), glVertex2f(0,glyph_height)
			glTexCoord2f(0,y), glVertex2f(0,0)
			glTexCoord2f(x,y), glVertex2f(glyph_width,0)
			glTexCoord2f(x,0), glVertex2f(glyph_width, glyph_height)
			glEnd()
			glPopMatrix()
	
			# Note, PIL's FreeType interface hides the advance from us.
			# Normal PIL clients are rendering an entire string through FreeType, not
			# a single character at a time like we are doing here.
			# Because the advance value is hidden from we will advance
			# the "pen" based upon the rendered glyph's width. This is imperfect.
			glTranslatef(glyph_width + 0.75, 0, 0)
	
			# //increment the raster position as if we were a bitmap font.
			# //(only needed if you want to calculate text length)
			# //glBitmap(0,0,0,0,face->glyph->advance.x >> 6,0,NULL)
	
			# //Finnish the display list
			glEndList()
	
		return

	# Get the length of a certain string by looking it up in our lookup table
	def getSize(self, string):
		return self.font.getsize(string)
	
	# /// A fairly straight forward function that pushes
	# /// a projection matrix that will make object world 
	# /// coordinates identical to window coordinates.
	@staticmethod
	def pushScreenCoordinateMatrix():
		glPushAttrib(GL_TRANSFORM_BIT)
		viewport = glGetIntegerv(GL_VIEWPORT)
		glMatrixMode(GL_PROJECTION)
		glPushMatrix()
		glLoadIdentity()
		gluOrtho2D(viewport[0],viewport[2],viewport[1],viewport[3])
		glPopAttrib()
		return
	
	
	# Pops the projection matrix without changing the current
	# MatrixMode.
	@staticmethod
	def pop_projection_matrix():
		glPushAttrib(GL_TRANSFORM_BIT)
		glMatrixMode(GL_PROJECTION)
		glPopMatrix()
		glPopAttrib()
		return
	
	# Alternative to draw. Same function definition as
	# FTGL fonts so zglText can be compatible with either
	#
	def draw (self, string):
		self.draw(0, 0, string)
	

	def draw (self, x, y, string):
		"""
		# ///Much like Nehe's glPrint function, but modified to work
		# ///with freetype fonts.
		"""
		# If There's No Text
		# Do Nothing
		if (string == None):
			return
		if (string == ""):
			return
			
		# We want a coordinate system where things coresponding to window pixels.
		zglFreeType.pushScreenCoordinateMatrix()
	
		# //We make the height about 1.5* that of
		h = float (self.m_font_height) / 0.63		
	

		# //Here is some code to split the text that we have been
		# //given into a set of lines.  
		# //This could be made much neater by using
		# //a regular expression library such as the one avliable from
		# //boost.org (I've only done it out by hand to avoid complicating
		# //this tutorial with unnecessary library dependencies).
		# //Note: python string object has convenience method for this :)
		lines = string.split ("\n")

		glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT | GL_TRANSFORM_BIT)
		glMatrixMode(GL_MODELVIEW)
		glDisable(GL_LIGHTING)
		glEnable(GL_TEXTURE_2D)
		glDisable(GL_DEPTH_TEST)
		glEnable(GL_BLEND)
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

		glListBase(self.m_list_base)
		modelview_matrix = glGetFloatv(GL_MODELVIEW_MATRIX)

		# //This is where the text display actually happens.
		# //For each line of text we reset the modelview matrix
		# //so that the line's text will start in the correct position.
		# //Notice that we need to reset the matrix, rather than just translating
		# //down by h. This is because when each character is
		# //draw it modifies the current matrix so that the next character
		# //will be drawn immediatly after it.  
		for i in xrange (len (lines)):
			line = lines [i]

			glPushMatrix ()
			glLoadIdentity ()
			glTranslatef (x,y-h*i,0);
			glMultMatrixf (modelview_matrix);

			# //  The commented out raster position stuff can be useful if you need to
			# //  know the length of the text that you are creating.
			# //  If you decide to use it make sure to also uncomment the glBitmap command
			# //  in make_dlist().
			# //	glRasterPos2f(0,0);
			glCallLists (line)
			# //	rpos = glGetFloatv (GL_CURRENT_RASTER_POSITION)
			# //	float len=x-rpos[0];
			glPopMatrix()

		glPopAttrib()
		zglFreeType.pop_projection_matrix()
		return

	def release (self):
		""" Release the gl resources for this Face.
			(This provides the functionality of KillFont () and font_data::clean ()
		"""
		if (self.m_allocated):
			# Free up the glTextures and the display lists for our face
			glDeleteLists ( self.m_list_base, 128);
			for ID in self.textures:
				glDeleteTextures (ID);
			# Extra defensive. Clients that continue to try and use this object
			# will now trigger exceptions.
			self.list_base = None
			self.m_allocated = False
		self.font = None
		return

	def __del__ (self):
		""" Python destructor for when no more refs to this Face object """
		self.release ()
		return


# Unit Test harness if this python module is run directly.
if __name__ == "__main__":
	print "testing availability of freetype font\n"
#	font_name = "arial.ttf"
	font_name = "FreeUniversal-Regular.ttf"
#	ft = ImageFont.load_default()
	ft = ImageFont.truetype(font_name, 15)
	if ft:
		print "Found the TrueType font "+font_name
	else:
		print "faild to find the TrueTYpe font 'arial'\n"



from OpenGL.GL import *
from OpenGL.GLU import *
from zglUtils import *

class zglCamera:
	def __init__(self):
		# Camera variables
		self.position = [0.0, 0.0, -1.0]
		self.lookat = [0.0, 0.0, 0.0]
		self.up = [0.0, 1.0, 0.0]
		
#	def setPosition(self, *args):
#		self.position		

	def setView(self):
		glMatrixMode(GL_MODELVIEW)
		glLoadIdentity()
		gluLookAt(	self.position[0], self.position[1], self.position[2],	# Camera position
					self.lookat[0], self.lookat[1], self.lookat[2],	# Camera look at
					self.up[0], self.up[1], self.up[2] )	# Up vector


class zglPerspective:
	def __init__(self):
		self.fov = 45.0
		self.width = 640.0
		self.height = 480.0
		self.near = 1.0
		self.far = 100.0
		
	def setProjection(self):
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(self.fov, 1.0*self.width/self.height, self.near, self.far)



# A class that defines an infinite plane, given by the equation
# 		ax + by + cz + d = 0
#
class zglPlane:
	def __init__(self, a, b, c, d):
		self.a = a
		self.b = b
		self.c = c
		self.d = d
		
	@staticmethod
	def fromNormalAndPoint(normal, point):
		# normal and point are both assumed to be 3-element lists
		d = -sum( zglVector.multiply(normal, point) )
		
		return zglPlane(normal[0], normal[1], normal[2], d)
		
	# Distance of a point from the line
	def distance(self, point):
		return self.a*point[0] + self.b*point[1] + self.c*point[2] + self.d
		
	
	

# A class that takes in a zglCamera and zglPerspective
# and generates the view frustum planes and tests objects
# whether they are within the 
#
class zglViewFrustumCulling:
	def __init__(self, perspective, camera):
		self.perspective = perspective
		self.camera = camera
		
		self.planes_initialized = False
		self.test_nearfar = False
		
	
	def update(self):
		self.updatePlanes()
		
	# Calculate the 6 planes that define the current view frustum
	def updatePlanes(self):
		camera_position = self.camera.position
		
		# unit direction vector of the camera direction and up vector
		camera_direction = zglVector.normalize( zglVector.subtract(self.camera.lookat, self.camera.position) )
		camera_up = zglVector.normalize(self.camera.up)
		
		# Top, bottom, left, right planes can be found simply using
		# the camera lookat vector, which is by definition in the center of 
		# the frustum.
		
		# First we need a vector pointing to the right:
		# This is simply the cross product of the camera's direction vector and its up vector
		camera_right = zglVector.crossProduct(camera_direction, camera_up)

		# Calculate the normals of each plane		
		#
		# we need one plane (either far or near) to generate normals for the 4 surround planes
		# so may as well use the far plane here:
		far_height = 2 * math.tan(self.perspective.fov/2) * self.perspective.far
		far_width = far_height * self.perspective.width/self.perspective.height
		
		# Center point for the far plane
		far_center = zglVector.add( self.camera.position, zglVector.multiplyConstant(camera_direction, self.perspective.far) )
		
		# Use the right edge of the far plane as reference point
		# to create a unit vector that lies along the right plane
		far_right = map(lambda i,j,k:  i + j*far_width/2 - k,
						far_center, camera_right, camera_position)
		far_right = zglVector.normalize(far_right)
		
		# Now we know both the far_right and the camera up vector are both on the right plane
		# So the normal is then simply the cross product of the two.
		# Note: take care with the order of the cross product to get the normal to
		# correctly point inwards.
		right_plane_normal = zglVector.crossProduct(camera_up, far_right)
		
		# Now do the same for the 3 other planes
		far_left = map(lambda i,j,k:  i - j*far_width/2 - k,
						far_center, camera_right, camera_position)
		far_left = zglVector.normalize(far_left)
		left_plane_normal = zglVector.crossProduct(far_left, camera_up)
		
		far_top = map(lambda i,j,k:  i + j*far_height/2 - k,
						far_center, camera_up, camera_position)
		far_top = zglVector.normalize(far_top)
		top_plane_normal = zglVector.crossProduct(far_top, camera_right)
		
		far_bottom = map(lambda i,j,k:  i - j*far_height/2 - k,
						far_center, camera_up, camera_position)
		far_bottom = zglVector.normalize(far_bottom)
		bottom_plane_normal = zglVector.crossProduct(camera_right, far_bottom)
		
		# Now we can create the planes!
		#
		self.top_plane = zglPlane.fromNormalAndPoint(top_plane_normal, self.camera.position)
		self.bottom_plane = zglPlane.fromNormalAndPoint(bottom_plane_normal, self.camera.position)
		self.left_plane = zglPlane.fromNormalAndPoint(left_plane_normal, self.camera.position)
		self.right_plane = zglPlane.fromNormalAndPoint(right_plane_normal, self.camera.position)
		
		self.planes_initialized = True
		
		if self.test_nearfar:
			# Now for Near and Far planes
			#
			# Near and far plane size
			near_height = 2 * math.tan(self.perspective.fov/2) * self.perspective.near
			near_width = near_height * self.perspective.width/self.perspective.height
			
			near_center = zglVector.add( self.camera.position, zglVector.multiplyConstant(camera_direction, self.perspective.near) )
	
			# Near plane points into the frustum (same as camera direction)
			self.near_plane = zglPlane.fromNormalAndPoint(camera_direction, near_center)
			# Far plane points towards the camera (negative camera direction)
			self.far_plane = zglPlane.fromNormalAndPoint(map(lambda i: -i, camera_direction), far_center)
		else:
			self.near_plane = None
			self.far_plane = None
	
	
	# Calculate planes using a matrix
	# The matrix has to be the ModelviewMatrix * ProjectionMatrix
	# (the order which OpenGL multiplies a vertex)
	#
	# The matrix is assumed to be column-major format (OpenGL standard)
	#
	# see: http://www.lighthouse3d.com/opengl/viewfrustum/index.php?clipspace
	def updatePlanesUsingMatrix(self, m):
		#	0	4	8	12
		#	1	5	9	13
		#	2	6	10	14
		#	3	7	11	15
		
		# Left Plane: col1 + col4
		self.left_plane = zglPlane( m[0] + m[12],
									m[1] + m[13],
									m[2] + m[14],
									m[3] + m[15] )
		# Right Plane: -col1 + col4
		self.right_plane = zglPlane( -m[0] + m[12],
									-m[1] + m[13],
									-m[2] + m[14],
									-m[3] + m[15] )
		# Top Plane: col2 + col4
		self.top_plane = zglPlane( m[4] + m[12],
									m[5] + m[13],
									m[6] + m[14],
									m[7] + m[15] )
		# Bottom Plane: -col2 + col4
		self.bottom_plane = zglPlane( -m[4] + m[12],
									-m[5] + m[13],
									-m[6] + m[14],
									-m[7] + m[15] )
		if self.test_nearfar:
			# Near plane: Col3 + Col4
			self.near_plane = zglPlane( m[8] + m[12],
										m[9] + m[13],
										m[10] + m[14],
										m[11] + m[15] )
			# Far plane: -Col3 + Col4
			self.far_plane = zglPlane( -m[8] + m[12],
										-m[9] + m[13],
										-m[10] + m[14],
										-m[11] + m[15] )
		else:
			self.near_plane = None
			self.far_plane = None

		
		
	# Test a point to see if we're inside the frustum
	# All plane normals are pointing inside the frustum, so the
	# distance test should all be positive for the point to
	# be inside
	#
	def containsPoint(self, point):
		if not self.planes_initialized:
			return True
	
		if (self.top_plane.distance(point) > 0
			and self.bottom_plane.distance(point) > 0
			and self.left_plane.distance(point) > 0
			and self.right_plane.distance(point) > 0
			
			# The above are the required tests. Sometimes we want to save
			# time and don't bother with near/far planes, because our scene
			# is wide/tall rather than deep
			and (self.near_plane is None or self.near_plane.distance(point) > 0)
			and (self.far_plane is None or self.far_plane.distance(point) > 0)
			):
			return True
			
		# One of the test failed
		return False
	
	
	def containsSphere(self, point, radius):
		if not self.planes_initialized:
			return True
			
 		if (self.top_plane.distance(point) > -radius
			and self.bottom_plane.distance(point) > -radius
			and self.left_plane.distance(point) > -radius
			and self.right_plane.distance(point) > -radius
			
			# The above are the required tests. Sometimes we want to save
			# time and don't bother with near/far planes, because our scene
			# is wide/tall rather than deep
			and (self.near_plane is None or self.near_plane.distance(point) > -radius)
			and (self.far_plane is None or self.far_plane.distance(point) > -radius)
			):
			return True
			
		# One of the test failed
		return False
		
	
	# Assumed to be a zglObject
	# and the object is assumed to have
	def containsObject(self, object):
		return self.containsSphere(object.position, object.getBoundingRadius())



#	Instead  of using planes to do culling
#	The radar approach is more efficient
#
class zglViewFrustumRadarCulling:
	def __init__(self, perspective, camera):
		self.perspective = perspective
		self.camera = camera
			
	# When perspective or camera has changed
	def update(self):
		self.z_axis = zglVector.normalize( zglVector.subtract(self.camera.lookat, self.camera.position) )
		self.x_axis = zglVector.normalize( zglVector.crossProduct(self.z_axis, self.camera.up) )
		self.y_axis = zglVector.crossProduct( self.x_axis, self.z_axis )
		
		# convert to radians
		fov = PI_OVER_180 * self.perspective.fov
		
		self.tan_fov = math.tan(fov/2)
		self.ratio = float(self.perspective.width)/self.perspective.height
		
		# for sphere test, need to multiply width and height by this to be
		# accurate. See: http://www.lighthouse3d.com/opengl/viewfrustum/index.php?camspace3
		self.sphere_factor_y = 1.0 / math.cos(fov)
		
		fov_x = math.atan(self.tan_fov * self.ratio)
		self.sphere_factor_x = 1.0 / math.cos(fov_x)

#		print "ratio = "+str(self.ratio)+" w,h="+str(self.perspective.width)+","+str(self.perspective.height)
#		print "fov = "+str(fov)+"  fov_x = "+str(fov_x)
#		print "sphere_factor_y = "+str(self.sphere_factor_y)+"  sphere_factor_x = "+str(self.sphere_factor_x)
		

	def containsPoint(self, point):
		# Vector from camera to point in question
		v = zglVector.subtract(point, self.camera.position)
		
		# Compute and test the z coordinate of the point, in the camera's frame of reference
		z = zglVector.dotProduct(v, self.z_axis)
		if z < self.perspective.near or z > self.perspective.far:
			return False
				
		# Using the z coordinate, we can then calculate the width and height of the frustum
		# at the current point
		half_height = z * self.tan_fov
		
		y = zglVector.dotProduct(v, self.y_axis)
		if y < -half_height or y > half_height:
			return False
		
		half_width = half_height * self.ratio
			
		x = zglVector.dotProduct(v, self.x_axis)
		if x < -half_width or x > half_width:
			return False
			
		return True
		
	
	def containsSphere(self, point, radius):
		# Vector from camera to point in question
		v = zglVector.subtract(point, self.camera.position)
		
		# Compute and test the z coordinate of the point, in the camera's frame of reference
		z = zglVector.dotProduct(v, self.z_axis)
		if z < self.perspective.near - radius or z > self.perspective.far + radius:
			return False
				
		# Using the z coordinate, we can then calculate the width and height of the frustum
		# at the current point
		half_height = z * self.tan_fov
		
		y = zglVector.dotProduct(v, self.y_axis)
		adjusted_radius = radius * self.sphere_factor_y
		if y < -half_height-adjusted_radius or y > half_height+adjusted_radius:
			return False
		
		half_width = half_height * self.ratio
			
		x = zglVector.dotProduct(v, self.x_axis)
		adjusted_radius = radius * self.sphere_factor_x
		if x < -half_width-adjusted_radius or x > half_width+adjusted_radius:
			return False
			
		return True
		
	# Assumed to be a zglObject
	# and the object is assumed to have
	def containsObject(self, object):
		return self.containsSphere(object.position, object.getBoundingRadius())




import os
import sys
import unittest

APP_ROOT = os.getenv('ZGL_HOME')
sys.path.insert(0, '%s/src' % APP_ROOT)

import wx

import zgl_graphdrawer.Application as ApplicationModule
import zgl_graphdrawer.canvas.wx_canvas as CanvasModule
import zgl_graphdrawer.Frame as FrameModule
import zgl_graphdrawer.GraphDrawerManager as ContextManagerModule
import zgl_graphdrawer.utils as UtilsModule

import MockData as DataModule

class TestCase(unittest.TestCase):

    def setUp(self):
        print "setUp"
        pass

    def tearDown(self):
        print "tearDown"
        pass

    def _testApplication1(self):
        app = wx.PySimpleApp()
        frame = wx.Frame(None, title='Hello World')
        frame.Show()
        app.MainLoop()


    def testApplication2(self):
        print "starting"

        app = ApplicationModule.wxApplication()
        app.setResourcePath(
            os.path.sep.join([os.getenv('ZGL_HOME'), 'resources']))


        frame = FrameModule.wxFrame(
            None, title='foo',
            size=wx.Size(ApplicationModule.DEFAULT_WIDTH,
                         ApplicationModule.DEFAULT_HEIGHT))
        frame.app(app)
        canvas = CanvasModule.Canvas(frame, 
                                     ApplicationModule.DEFAULT_WIDTH,
                                     ApplicationModule.DEFAULT_HEIGHT)

        contextManager = ContextManagerModule.GraphDrawerManager()	
        contextManager.initialize(canvas=canvas)

        app.contextManager(contextManager)
        contextManager.app(app)

        # TODO:
        # this needs to be specified elsewhere
        contextManager.commandPath('dot', '/sw/bin/dot')
        
        app.initializePolicies()

        (mockNodes, mockEdges) = DataModule.createMockData()
        contextManager.setDataContext(mockNodes, mockEdges)

        canvas.computeLayout()

        frame.Show()
        app.MainLoop()

        print "ending"
        pass

    # END class TestApplication1
    pass



def main():
    UtilsModule.configLogging()

    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCase, 'test'))

    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




import logging
import sys
import time

import wx

class Menu(object):
    
    def __init__(self, event=None, parent=None, canvas=None):

        self.parent = parent
        self.canvas = canvas
        self.event = event
        return
    
    def addMenuItem(self, label, callback):
        menuitem = wx.MenuItem(self, wx.NewId(), label)
        self.AppendItem(menuitem)
        self.Bind(wx.EVT_MENU, callback, id=menuitem.GetId())
        return
    
    def bindEvents(self):
        raise NotImplementedError

    pass


class SelectionContextualMenu(wx.Menu, Menu):
    
    def __init__(self, event, parent, canvas, objects):
        wx.Menu.__init__(self)
        Menu.__init__(self, event, parent, canvas)
        self.objects = objects
        return

    # END class SelectionContextualMenu
    pass


class CanvasContextualMenu(wx.Menu, Menu):
    
    def __init__(self, *args, **kwds):
        wx.Menu.__init__(self)

        Menu.__init__(self, *args, **kwds)
        return

    def bindEvents(self):
        self.addMenuItem('Create node', self.OnCreateNode)
        return
    
    def OnCreateNode(self, event):
        print "should create node"
        return

    # END class CanvasContextualMenu
    pass




import copy
import os

class VisualPolicy(object):

    KEY_SELECTION_TRUE = "selected"
    KEY_SELECTION_FALSE = "not selected"


    COLOUR_DEFAULT = [1.0, 1.0, 1.0, 1.0]

    FONT_DEFAULT = "FreeUniversal-Regular"
    FONT_DEFAULT_FILE = "%s.ttf" % FONT_DEFAULT

    KEY_FONT_DEFAULT = "default font"    



    def __init__(self, contextManager):
        self.contextManager(contextManager)
        self._backgroundMap = {}
        self._textMap = {}
        self._colourMap = {}
        self._fontMap = {}
        return

    def contextManager(self, value=None):
        if value is not None:
            self._contextManager = value
        # being set in the constructor
        # so no need to verify if there's a context manager
        return self._contextManager

    def colour(self, key, value=None):
        if value is not None:
            self._colourMap[key] = value
        if not key in self._colourMap:
            self._colourMap[key] = copy.copy(VisualPolicy.COLOUR_DEFAULT)
        return self._colourMap[key]

    def getFontPath(self):
        return self.contextManager().resourcePath()

    def getPathForDefaultFont(self):
        fontPath = os.sep.join([self.getFontPath(),
                                VisualPolicy.FONT_DEFAULT_FILE])
        return fontPath


    def font(self, key, value=None):
        if value is not None:
            self._fontMap[key] = value
        if not key in self._fontMap:
            self._fontMap[key] = copy.copy(VisualPolicy.FONT_DEFAULT)
        return self._fontMap[key]


    # END class VisualPolicy
    pass



import logging
import math, sys, os, time


import OpenGL
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *


import zgl.zglUtils as zglUtilsModule
import zgl.zglApplication as ZglApplicationModule
from zgl.zglPrimitives import zglMouse




from zgl_graphdrawer.EdgePolicy import *
from zgl_graphdrawer.GraphDrawerManager import GraphDrawerManager
from zgl_graphdrawer.LayoutPolicy import *
from zgl_graphdrawer.NodePolicy import *
from zgl_graphdrawer.PortPolicy import *
from zgl_graphdrawer.VisualPolicy import *

DEFAULT_WIDTH = 800
DEFAULT_HEIGHT = 600



class Application(object):
    
    def __init__(self):
        self._resourceValues = {}
        return



    def setResourceValue(self, key, value):
        self._resourceValues[key] = value
        return

    def getResourceValue(self, key, default=None):
        if not key in self._resourceValues:
            return default
        return self._resourceValues[key]

    def setDefaultResourcePath(self):
        cwd = os.getcwd()
        resourcePath = os.path.sep.join(
            os.getenv('ZGL_HOME').split('/')+['resources'])
        self.setResourcePath(resourcePath)
        return

    def setResourcePath(self, resourcePath):
        if not os.path.exists(resourcePath):
            raise IOError('resource path %s does not exist' % resourcePath)
        self._resourcePath = resourcePath
        return

    def contextManager(self, value=None):
        if value is not None:
            self.graphDrawerManager = value
        return self.graphDrawerManager


    def initializePolicies(self):

        contextManager = self.contextManager()

        visualPolicy = self.initializeVisualPolicy(contextManager)
        nodePolicy = self.initializeNodePolicy(contextManager)
        edgePolicy = self.initializeEdgePolicy(contextManager)
        layoutPolicy = self.initializeLayoutPolicy(contextManager)
        portPolicy = self.initializePortPolicy(contextManager)
        nodePolicy.portPolicy(portPolicy)
        return
        
        
    def initializeEdgePolicy(self, contextManager):
        canvas = contextManager.canvas
        
        # create and set the edge policy
        key = 'edge policy'
        classObject = self.getResourceValue(
            "%s class" % key, default=SimpleVerticalEdgePolicy)
        edgePolicy = classObject()
        canvas.edgePolicy(edgePolicy)

        return edgePolicy
    
    def initializeLayoutPolicy(self, contextManager):
        canvas = contextManager.canvas
        
        # create and set the layout policy
        key = 'layout policy'
        classObject = self.getResourceValue(
            "%s class" % key, default=GraphVizLayoutPolicy)
        layoutPolicy = classObject(contextManager)
        canvas.layoutPolicy(layoutPolicy)

        return layoutPolicy
    
    def initializeNodePolicy(self, contextManager):
        canvas = contextManager.canvas

        # create the node policy for this application
        key = 'node policy'
        classObject = self.getResourceValue(
            "%s class" % key, default=SimpleNodePolicy)
        nodePolicy = classObject(contextManager)

        # set the visual policy
        visualPolicy = canvas.visualPolicy()
        nodePolicy.visualPolicy(visualPolicy)

        # set the colors for the node policy
        # we can instead set it on the visual policy first
        nodePolicy.initializeColours()
        for key, value in [
            (NodePolicy.KEY_BORDER_SELECTION_TRUE,  [1.0, 1.0, 1.0, 1.0]),
            (NodePolicy.KEY_BORDER_SELECTION_FALSE,  [0.6, 0.6, 0.6, 1.0]),
            (NodePolicy.KEY_BACKGROUND_SELECTION_TRUE,  [0.4, 0.4, 0.6, 1.0]),
            (NodePolicy.KEY_BACKGROUND_SELECTION_FALSE,  [0.3, 0.3, 0.5, 0.8]),
            (NodePolicy.KEY_TEXT_SELECTION_TRUE, [1.0, 1.0, 1.0, 1.0]),
            (NodePolicy.KEY_TEXT_SELECTION_FALSE, [0.8, 0.8, 0.8, 1.0])]:

            nodePolicy.colour(key, value)
            pass

        # set the node policy
        canvas.nodePolicy(nodePolicy)
        return nodePolicy
    
    
    
    def initializeVisualPolicy(self, contextManager):

        # create the visual policy
        key = 'visual policy'
        classObject = self.getResourceValue(
            "%s class" % key, default=VisualPolicy)
        visualPolicy = classObject(contextManager)
            
        # seupt the default font
        pathForDefaultFont = visualPolicy.getPathForDefaultFont()
        font = TextModule.zglFontManager.getFont(pathForDefaultFont, 16)
        visualPolicy.font(VisualPolicy.KEY_FONT_DEFAULT, font)
        
        canvas = contextManager.canvas

        # set the visual policy
        canvas.visualPolicy(visualPolicy)
        
        return visualPolicy
    
    def initializePortPolicy(self, contextManager):

        key = 'port policy'
        classObject = self.getResourceValue(
            "%s class" % key, default=PortPolicy)
        portPolicy = classObject(contextManager)

        canvas = contextManager.canvas
        canvas.portPolicy(portPolicy)
        return portPolicy

    
    # END class Application
    pass




class zglApplication(Application, ZglApplicationModule.zglApplication):
    mouse_drag = False
    last_mouse_x = 0
    last_mouse_y = 0

    def __init__(self, name=None, 
                 width=DEFAULT_WIDTH, height=DEFAULT_HEIGHT):

        if name is None:
            name = 'Application'

        ZglApplicationModule.zglApplication.__init__(
            self, width=width, height=height, app_name=name)
        Application.__init__(self)

        glClearColor(0.1, 0.1, 0.2, 0.5)

        return

    def initGL(self):
        ZglApplicationModule.zglApplication.initGL(self)

        # Enable blending
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST) # Really Nice Perspective Calculations

        glShadeModel(GL_SMOOTH)				# Enables Smooth Color Shading
        glClearDepth(1.0)					# Enables Clearing Of The Depth Buffer

        glEnable(GL_DEPTH_TEST)				# Enables Depth Testing
        glDepthFunc(GL_LEQUAL)				# The Type Of Depth Test To Do

        glDisable(GL_LIGHTING)
        glColor3f(1.0, 1.0, 1.0)

        self.updateProjection()
        return

    
    def updateProjection(self):
        """ Called by reshapeFunc when window size has changed """
        self.updateProjection2DOrtho()

    def reshapeFunc(self, width, height):
        ZglApplicationModule.zglApplication.reshapeFunc(self, width, height)
        self.contextManager().updateFrame((0.0, 0.0, self.width, self.height))


    def displayFunc(self):
        glDrawBuffer(GL_BACK)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        glDisable(GL_LIGHTING)
        glColor3f(1.0, 1.0, 1.0)

        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()

        """main drawing stuff goes here"""
        glPushMatrix()
        self.contextManager().step()
        self.contextManager().draw()
        glPopMatrix()

        x = 30
        y = 30
        glColor3f(1.0, 1.0, 1.0)
        self.drawFPS()

        glutSwapBuffers()

        return

    
    def drawFPS(self):
        if hasattr(self, "last_t"):
            # self.fps = 1.0/float(time.time()-self.last_t)
            self.fps = float(time.time()-self.last_t)
            
            glColor3f(1.0, 1.0, 1.0)
            zglUtilsModule.zglUtils.drawBitmapText(
                "FPS: %f" % (self.fps), 
                # x, y
                30, self.height-30, 
                font=GLUT_BITMAP_HELVETICA_10)
        self.last_t = time.time()

        return


    def keyboardFunc(self, key, x, y):
        # If escape is pressed, kill everything.
        if key == '\033':
            logging.debug("Quitting!")
            sys.exit()
        else:
            self.contextManager().keyDown(key)

    def keyboardUpFunc(self, key, x, y):
        self.contextManager().keyUp(key)

    def specialFunc(self, *args):
        key = args[0]
        if key == GLUT_KEY_UP:
            logging.debug("Up!   GLUT_KEY_UP=" + str(GLUT_KEY_UP))
        elif key == GLUT_KEY_DOWN:
            logging.debug("Down!   GLUT_KEY_DOWN=" + str(GLUT_KEY_DOWN))
        elif key == GLUT_KEY_LEFT:
            logging.debug("Left!   GLUT_KEY_LEFT=" + str(GLUT_KEY_LEFT))
        elif key == GLUT_KEY_RIGHT:
            logging.debug("Right!   GLUT_KEY_RIGHT=" + str(GLUT_KEY_RIGHT))

    def mousePassiveFunc(self, x, y):
        zglMouse.setPosition(x, y)

        self.last_mouse_x = x
        self.last_mouse_y = y
        self.contextManager().mouseMoved(x, self.height-y)

    def mouseFunc(self, button, state, x, y):
        if state == GLUT_DOWN:
            self.contextManager().mouseButtonDown(button, x, self.height-y)
        elif state == GLUT_UP:
            self.contextManager().mouseButtonUp(button, x, self.height-y)

    def mouseMotionFunc(self, x, y):
        self.contextManager().mouseDragged(x, self.height-y)

        return

    # END class Application
    pass



import logging

def configLogging():
    """
    this will be called by all the main functions 
    to use the default setup for logging
    """
    # define a basic logger to write to file
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(levelname)-8s %(message)s',
                        datefmt='%a, %d %b %Y %H:%M:%S',
                        filename='/tmp/canvas.log',
                        filemode='w')

    # define a handler to write to stderr
    # console = logging.StreamHandler()
    # set the level of this to verbosity of severity 'warning'
    # console.setLevel(logging.WARNING)
    # set a format which is simpler for console use
    # formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
    # tell the handler to use this format
    # console.setFormatter(formatter)
    # add the handler to the root logger
    # logging.getLogger('').addHandler(console)

    # end def configureLogging
    pass



#!/usr/bin/python

import os
import sys

APP_ROOT = os.getenv('ZGL_HOME')
sys.path.insert(0, '%s/src' % APP_ROOT)

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from zgl.zglText import *

font = None
width = 600
height = 600

def do_ortho():
    w, h = width, height
    glViewport(0, 0, w, h)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    size = max(w, h) / 2.0
    aspect = float(w) / float(h)
    if w <= h:
        aspect = float(h) / float(w)
        glOrtho(-size, size, -size*aspect, size*aspect, -100000.0, 100000.0)
    else:
        glOrtho(-size*aspect, size*aspect, -size, size, -100000.0, 100000.0)
    glScaled(aspect, aspect, 1.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def draw_scene():
    
    glColor3f(1.0, 1.0, 1.0)
    
    text = ["Hello", "This is a test", "Of zglText using FTGL"]
    for i in range(len(text)):
        line = text[i]
        text1 = font.draw(line);
        glTranslatef(0.0, - font.getSize(line)[1] - 5, 0.0)
            
  
def on_display():
    glClear(GL_COLOR_BUFFER_BIT)
    do_ortho()
    draw_scene()
    glutSwapBuffers()

def on_reshape(w, h):
    width, height = w, h

def on_key(key, x, y):
    if key == '\x1b':
        sys.exit(1)

if __name__ == '__main__':
    glutInitWindowSize(width, height)
    glutInit(sys.argv)
    glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE)
    glutCreateWindow("zglText FTGL Test")
    glClearColor(0.0, 0.0, 0.0, 0.0)

    try:
        font = zglFTGLFont(sys.argv[1], 24)
    except:
        print "usage:", sys.argv[0], "font_filename.ttf"
        sys.exit(0)

    glutDisplayFunc(on_display)
    glutReshapeFunc(on_reshape)
    glutKeyboardUpFunc(on_key)

    glutMainLoop()



import Image
from numpy import *
from OpenGL.GL import *
from zglUtils import *
from zglDrawer import *


# Static class for storing mouse information
#
class zglMouse(object):
    # Current and last buttons
    x = 0
    y = 0
    last_x = 0
    last_y = 0

    # Buttons
    left_button = 0
    middle_button = 0
    right_button = 0

    @staticmethod
    def setPosition(x, y):
        zglMouse.last_x = zglMouse.x
        zglMouse.last_y = zglMouse.y
        zglMouse.x = x
        zglMouse.y = y

    @staticmethod
    def mouseFunc(button, state, x, y):
        if button == GLUT_LEFT_BUTTON:
            if state == GLUT_DOWN:
                zglMouse.left_button = 1
            elif state == GLUT_UP:
                zglMouse.left_button = 0
        if button == GLUT_RIGHT_BUTTON:
            if state == GLUT_DOWN:
                zglMouse.right_button = 1
            elif state == GLUT_UP:
                zglMouse.right_button = 0
        if button == GLUT_MIDDLE_BUTTON:
            if state == GLUT_DOWN:
                zglMouse.middle_button = 1
            else:
                zglMouse.middle_button = 0

    @staticmethod
    def getDelta():
        return (zglMouse.x - zglMouse.last_x, zglMouse.y - zglMouse.last_y)

# general zglObject
#
class zglObject(object):
    def __init__(self):
        self.position = [0, 0, 0]
        self.rotation = [0, 0, 0]
        
        # If a quaternion is set, use it instead of self.rotation for the rotation
        self.quaternion = None		
        self.size = [1.0, 1.0, 1.0]
        self.colour = [1.0, 1.0, 1.0, 1.0]
        self.alpha = 1.0

        # Display list support
        self.useDisplayList = True
        self.displayList = -1

        # I wanted to use numpy for the array stuff but 
        # for some reason it seems a little bit slower
#		self.position = array([0, 0, 0])
#		self.rotation = array([0, 0, 0])
#		self.quaternion = None		# If a quaternion is set, use it instead of self.rotation for the rotation
#		self.size = array([1.0, 1.0, 1.0])
#		self.colour = array([1.0, 1.0, 1.0, 1.0])
#		self.alpha = 1.0

    # This function is defined because objects with multiple children objects
    # should inherit the parent's alpha. This function facilitates that ...
    # thus all zglObjects should have one even a trivial one to not cause errors
    def setAlpha(self, alpha):
        self.alpha = alpha

    def useColour(self, colour):
        if colour is not None:
            glColor4f(colour[0], colour[1], colour[2], colour[3])
            
    def useColourWithAlpha(self, colour, alpha):
        if colour is not None:
            glColor4f(colour[0], colour[1], colour[2], colour[3]*alpha)

    def doTranslation(self):
        glTranslate(self.position[0], self.position[1], self.position[2])

    def predraw(self):
        glPushMatrix()
        glTranslate(self.position[0], self.position[1], self.position[2])
        if self.quaternion is not None:
#			x, y, z, r = self.quaternion.XYZR()
#			glRotate(r*3.1415, x, y, z)
            glMultMatrixf( self.quaternion.glMatrix() )
        else:
            glRotate(self.rotation[0], 1, 0, 0)
            glRotate(self.rotation[1], 0, 1, 0)
            glRotate(self.rotation[2], 0, 0, 1)
            pass

        #glColor4f(self.colour[0], self.colour[1], self.colour[2], self.colour[3]*self.alpha)
        self.useColourWithAlpha(self.colour, self.alpha)

        return

    def postdraw(self):
        glPopMatrix()

    def maindraw(self):
        # This is where the main GL draw code goes
        pass

    def draw(self):

        if not self.visible():
            return
        
        self.predraw()

        if glIsList(self.displayList):
            glCallList(self.displayList)
        else:
            if self.useDisplayList:
                self.displayList = glGenLists(1)

                glNewList(self.displayList, GL_COMPILE_AND_EXECUTE)
                self.maindraw()
                glEndList()
            else:
                self.maindraw()

        self.postdraw()
        return


    # Return the radius of the sphere that bounds this object
    # subclasses will have to override this if self.size it
    # not accurate
    #
    def getBoundingRadius(self):
        return max(self.size)

    def visible(self, value=None):
        if value is not None:
            self._visible = value
        if not hasattr(self, '_visible'):
            self._visible = True
        return self._visible
    
    # END class zglObject
    pass


class zglLine(zglObject):

    def __init__(self):
        zglObject.__init__(self)
        self.points = []

    def maindraw(self):
        if len(self.points) < 2:
            return
        self.useColour(self.colour)
        zglDrawer.drawLine(self.points)
        return

    # END class zglLine
    pass



class zglRect(zglObject):
    
    def __init__(self):
        zglObject.__init__(self)
        self.borderColour = None
        
        # whether or not to use position as bottom-left corner of rect
        self.corner_mode = False
        
        self.use_texture = False
        return


    def setTexture(self, texture_id, texture_coords):
        self.texture_id = texture_id
        self.texture_coords = texture_coords
        self.use_texture = True

    def setTextureImage(self, image, mode='RGBX'):
        ix = image.size[0]
        iy = image.size[1]
        image = image.tostring("raw", mode, 0, -1)

        # Create Texture
        new_texture_id = glGenTextures(1)
        self.texture_id = new_texture_id
        glBindTexture(GL_TEXTURE_2D, self.texture_id)
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
        glTexImage2D(GL_TEXTURE_2D, 0, 3, ix, iy, 0, GL_RGBA, GL_UNSIGNED_BYTE, image)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
        glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL)

        self.texture_coords = ( (0.0, 0.0), (1.0, 0.0), (1.0, 1.0), (0.0, 1.0) )
        self.use_texture = True

    def setTextureImageFile(self, texture_image_file, mode='RGBX'):
        image = Image.open(texture_image_file)
        self.setTextureImage(image, mode=mode)
        self.texture_coords = ( (0.0, 0.0), (1.0, 0.0), (1.0, 1.0), (0.0, 1.0) )
        # self.texture_id = zglUtils.loadTextureImage(texture_image_file)
        self._texture_image_file = texture_image_file
        return


    def maindraw(self):
        if not self.corner_mode:
            glTranslatef(-self.size[0]/2, -self.size[1]/2, 0)

        if self.use_texture:
            glEnable(GL_TEXTURE_2D)
            glBindTexture(GL_TEXTURE_2D, self.texture_id)
            glBegin(GL_QUADS);
            glNormal3f(0.0, 0.0, 1.0);
            glTexCoord2f(self.texture_coords[0][0], self.texture_coords[0][1])
            glVertex2f(0, 0)
            glTexCoord2f(self.texture_coords[1][0], self.texture_coords[1][1])
            glVertex2f(self.size[0], 0)
            glTexCoord2f(self.texture_coords[2][0], self.texture_coords[2][1])
            glVertex2f(self.size[0], self.size[1])
            glTexCoord2f(self.texture_coords[3][0], self.texture_coords[3][1])
            glVertex2f(0, self.size[1])
            glEnd()
            glDisable(GL_TEXTURE_2D)

        elif self.colour is not None:
            glBegin (GL_QUADS);
            glNormal3f(0.0, 0.0, 1.0);
            glVertex2f(0, 0)
            glVertex2f(self.size[0], 0)
            glVertex2f(self.size[0], self.size[1])
            glVertex2f(0, self.size[1])
            glEnd ()

        if self.borderColour is not None:
            self.useColour(self.borderColour)
            glBegin (GL_LINE_LOOP)
            glVertex2f(0, 0)
            glVertex2f(self.size[0], 0)
            glVertex2f(self.size[0], self.size[1])
            glVertex2f(0, self.size[1])
            glEnd ()

        return

    def drawStroke(self):
        self.predraw()
        if not self.corner_mode:
            glTranslatef(-self.size[0]/2, -self.size[1]/2, 0)
        glBegin (GL_LINE_LOOP)
        glVertex2f(0, 0)
        glVertex2f(self.size[0], 0)
        glVertex2f(self.size[0], self.size[1])
        glVertex2f(0, self.size[1])
        glEnd ()
        self.postdraw()

    def getBoundingRadius(self):
        return SQRT_2 * max(self.size)

    # END class zglRect
    pass



class zglEllipse(zglObject):
    def __init__(self):
        zglObject.__init__(self)
        self.borderColour = None
        self.corner_mode = False	# whether or not to use position as bottom-left corner of rect

    def maindraw(self):
        
        if not self.corner_mode:
            glTranslatef(-self.size[0]/2, -self.size[1]/2, 0)

        # we want at least 8 wedges
        # and for now, a max 
        smoothness = max(8, self.size[0])
        smoothness = min(smoothness, 360)
        
        x = self.size[0]/2.0
        y = self.size[1]/2.0
        
        # TODO:
        # radius is only for circle
        # need to calculate for an actual ellipse
        radius = self.size[0]/2.0
            
        if self.colour is not None:
            
            glBegin(GL_TRIANGLE_FAN)
            glNormal3f(0.0, 0.0, 1.0);
            for i in range(0, smoothness):    
                angle = i * math.pi * 2.0 / smoothness
                glVertex2f(x + radius * math.cos(angle), 
                           y + radius * math.sin(angle))
            glEnd() 

            
        if self.borderColour is not None:
            self.useColour(self.borderColour)
            
            glBegin(GL_LINE_LOOP)
            for i in range(0, smoothness):    
                angle = i * math.pi * 2.0 / smoothness
                glVertex2f(x + radius * math.cos(angle), 
                           y + radius * math.sin(angle))
            glEnd() 

        return
            
    def drawStroke(self):
        self.predraw()
        if not self.corner_mode:
            glTranslatef(-self.size[0]/2, -self.size[1]/2, 0)
        glBegin (GL_LINE_LOOP)
        glVertex2f(0, 0)
        glVertex2f(self.size[0], 0)
        glVertex2f(self.size[0], self.size[1])
        glVertex2f(0, self.size[1])
        glEnd ()
        self.postdraw()
        return

    # END class zglEllipse
    pass


class zglTriangle(zglObject):
    def __init__(self):
        zglObject.__init__(self)
        self.borderColour = None
        self.corner_mode = False	# whether or not to use position as bottom-left corner of rect
        return
    
    
    def maindraw(self):
        
        if not self.corner_mode:
            glTranslatef(-self.size[0]/2, -self.size[1]/2, 0)

        # we want at least 8 wedges
        # and for now, a max 
        smoothness = max(8, self.size[0])
        smoothness = min(smoothness, 360)
        
        x = self.size[0]/2.0
        y = self.size[1]/2.0
        
        # TODO:
        # radius is only for circle
        # need to calculate for an actual ellipse
        radius = self.size[0]/2.0
            
        if self.colour is not None:
            
            glBegin (GL_QUADS);
            glNormal3f(0.0, 0.0, 1.0);
            glVertex2f(0, 0)
            glVertex2f(self.size[0], 0)
            glVertex2f(self.size[0]/2, self.size[1])
            glEnd ()
            
            
        if self.borderColour is not None:
            self.useColour(self.borderColour)

            
            glBegin (GL_LINE_LOOP)
            glVertex2f(0, 0)
            glVertex2f(self.size[0], 0)
            glVertex2f(self.size[0]/2, self.size[1])
            glEnd ()

        return
            

    # END class Triangle
    pass







from OpenGL.GL import *
from zglFreeType import *
from zglPrimitives import *

import FTGL


ALIGN_LEFT = 'left'
ALIGN_RIGHT = 'right'
ALIGN_CENTER = 'center'

ALIGN_TOP = 'top'
ALIGN_MIDDLE = 'middle'
ALIGN_BOTTOM = 'bottom'


class zglText(zglObject):
		
	def __init__(self):
		zglObject.__init__(self)
		self.font = None
		self.text = ""
		self.horizontal_align = ALIGN_LEFT
		self.vertical_align = ALIGN_TOP
		return
	
	# update uses current font and text to calculate width and height
	def sizeToFit(self):
		width = 0
		height = 0
		if self.font is not None:
			if self.text is not "":
				width, height = self.font.getSize(self.text)
			else:
				height = self.font.m_font_height
		self.size[0] = width
		self.size[1] = height
	
	def getDrawingOffset(self):
		fontSize = self.font.getSize(self.text)
		xx = 0
		if self.horizontal_align is ALIGN_CENTER:		# CENTER ALIGN
			xx = (self.size[0]-fontSize[0])/2
		elif self.horizontal_align is ALIGN_RIGHT:	# RIGHT ALIGN
			xx = (self.size[0]-fontSize[0])
			
		yy = 0
		if self.vertical_align is ALIGN_MIDDLE:		# MIDDLE ALIGN
			yy = (self.size[1]-fontSize[1])/2
		elif self.vertical_align is ALIGN_BOTTOM:		# BOTTOM ALIGN
			yy = (self.size[1]-fontSize[1])

		return xx, yy

	def draw(self):
		if self.font is None: return
		
		self.predraw()
		

		xx, yy = self.getDrawingOffset()
		
		self.useColourWithAlpha(self.colour, self.alpha)
		
		glPushMatrix()
		glTranslate(xx, yy, 0)
		self.font.draw(self.text)
		glPopMatrix()
		
		self.postdraw()


# simple text box containing a single line of text with a desired background colour
#
class zglTextBox(zglText):
	def __init__(self):
		zglText.__init__(self)
		self.inner_margin = (2,5,2,5)	# top, right, down, left

		self.background = zglRect()
		self.background.corner_mode = True
		self.background.size = self.size	# Link the background's size to the overall size
		self.autoresize_x = True

	def sizeToFit(self):
		if self.autoresize_x:
			# If the textbox is autoresizing based on the text, adjust the background
			zglText.sizeToFit(self)
			self.size[0] += self.inner_margin[1] + self.inner_margin[3]
			self.size[1] += self.inner_margin[0] + self.inner_margin[2]

			# Now adjust the background position based on text alignment
			# Remember: zglRect's position is based on the center of the rect
			if self.horizontal_align is ALIGN_CENTER:
				self.background.position[0] = -self.size[0]/2-self.inner_margin[3]
			elif self.horizontal_align is ALIGN_RIGHT:
				self.background.position[0] = -self.size[0]-self.inner_margin[3]-self.inner_margin[1]
			else:
				self.background.position[0] = 0
				
			if self.vertical_align is ALIGN_TOP:
				self.background.position[1] = 0			
			elif self.vertical_align is ALIGN_MIDDLE:
				self.background.position[1] = -self.size[1]/2
			else:
				self.background.position[1] = -self.size[1]
	
		else:
			# Otherwise, we need to enforce the width/height on the 
			# NOT YET IMPLEMENTED
			pass
	
	def setAlpha(self, alpha):
		self.alpha = alpha
		self.background.alpha = alpha

	def draw(self):
		self.predraw()
		self.background.draw()
	
		# Now since this is now an extension of a zglObject
		# if we change x and y based on text alignment, that would
		# move the object errornously. So instead we calculate, 
		xx = 0
		yy = 0
		
		if self.horizontal_align is ALIGN_CENTER:
			xx = (self.size[0]-self.font.getSize(self.text)[0])/2
		elif self.horizontal_align is ALIGN_RIGHT:
			xx = (self.size[0]-self.font.getSize(self.text)[0])
			
			
		if self.vertical_align is ALIGN_BOTTOM:
			yy = -self.size[1]
		elif self.vertical_align is ALIGN_MIDDLE:
			yy = -self.size[1]/2

		glTranslatef(xx, yy, 0)

		self.useColourWithAlpha()
		self.font.draw(self.text)

		self.postdraw()
		
# NOT DONE!
#
class zglTextArea(zglObject):
	def __init__(self):
		self.font = None
		self.text = ""
		self.lines = []
		self.horizontal_align = ALIGN_LEFT
		self.vertical_align = ALIGN_TOP
		self.line_spacing = 12
	
	# Separate self.text into lines
# NOT DONE!
	def sizeToFit(self):
		self.lines = []
		current_word = ""
		current_line = ""
		newline = False
		for c in self.text:
			if c is '\n':
				
				newline = True
			else:
				current_word += c
		

	def draw(self):
		if self.font is None or len(self.lines) is 0: return
		
		self.predraw()
		
		yy = -(len(self.lines)-1) * self.line_spacing
		for line in self.lines:
			self.font.draw(0, yy, line)
			yy = yy + self.line_spacing
		
		self.postdraw()
		



# Support class that defines functions to deal with keypresses
# and mouse clicks
class zglTextInput:
	def keyPressed(self, key):
#		print "zglTextInput has entry: %s %d" % (key, ord(key))
#		if key is '\x0D':			# Enter
		if ord(key) is 13:
			self.actionEnter(self)
			self.text = ""
#		elif key is '\b':			# Backspace
		elif ord(key) is 8:
			self.text = self.text[:-1]
		elif key is '\033':		# ESC Key
			self.text = ""
		else:
			self.text += key
		self.sizeToFit()

	# When enter is pressed
	# override this for desired functionality
	def actionEnter(self):
		pass



class zglInputTextBox(zglTextBox, zglTextInput):
	def __init__(self):
		zglTextBox.__init__(self)

	def actionEnter(self):
		pass


class zglFTGLFont(FTGL.TextureFont):

	# Note: scale is here to render a sharper font than the
	# font size implies, such that when you zoom in it is still
	# sharp-looking
	
	def __init__(self, typeface, size, scale=2):
		FTGL.TextureFont.__init__(self, typeface)
		self.fontSize = size
		self.scale = scale
		self.FaceSize(size*scale, 288)
		
	def getSize(self, string):
		bbox = self.BBox(string)
		return ((bbox[3]-bbox[0])/self.scale, (bbox[4]-bbox[1])/self.scale)

	def draw(self, string):
		glPushMatrix()
		glScalef(1.0/self.scale, 1.0/self.scale, 1.0)
		self.Render(string)
		glPopMatrix()


class zglFontManager:

	USE_FTGL = 0
	USE_ZGLFREETYPE = 1
	
	mode = USE_FTGL
	library = dict()
	
	@staticmethod
	def getKey(typeface, size):
		return typeface+"_"+str(size)
		
	@staticmethod
	def createItem(typeface, size):
		if  zglFontManager.mode == zglFontManager.USE_FTGL:
			return zglFTGLFont(typeface, size)
		elif zglFontManager.mode == zglFontManager.USE_ZGLFREETYPE:
			return zglFreeType(typeface, size)
	
	@staticmethod
	def getFont(typeface, size):
		"""Gets a font from the existing created objects
		if it doesn't exist, it'll be created"""
		key = zglFontManager.getKey(typeface, size)
		if zglFontManager.library.has_key(key):
#			print "Font "+key+" found in library"
			return zglFontManager.library[key]
		else:
#			print "Font "+key+" not found in library, creating it"
			new_font = zglFontManager.createItem(typeface, size)
			if new_font is not None:
				zglFontManager.library[key] = new_font
				return zglFontManager.library[key]
			else:
				print "Cannot create font: %d" % (key)
		
		



import math

import zgl.zglPrimitives as PrimitivesModule

	
class EdgePolicy(object):
	"""
	this defines the API for EdgePolicy
	"""

	# damn British spelling
	def lineColour(self, value=None):
		"""
		TODO: modify this to make use of ColorPolicy
		"""
		if value is not None:
			self._lineColour = value
		return self._lineColour


	"""
	def setupEdge(self, edge):
		raise NotImplementedError
	"""


	def updateEdge(self, edge):
		raise NotImplementedError

	def createPath(self, inputPoint, outputPoint, steps):
		raise NotImplementedError

	# END class EdgePolicy
	pass


class SimpleEdgePolicy(EdgePolicy):

	def __init__(self):
		self.lineColour([1.0, 1.0, 1.0, 0.3])
		return
	
	"""
	def setupEdge(self, edge):
		edge.children = []
		if edge.inputNode is None or edge.outputNode is None:
			return
		
		point1 = (edge.inputNode.x + edge.inputNode.width, edge.inputNode.y + edge.inputNode.height/2)
		point2 = (edge.outputNode.x, edge.outputNode.y + edge.outputNode.height/2)
		
		line = PrimitivesModule.zglLine()
		line.points = self.createPath(point1, point2, 30)
		line.position = (0.0, 0.0, -0.01)
		edge.children.append(line)
		return
	"""
	
	def updateEdge(self, edge):
		pass

	
	def createPath(self, inputPoint, outputPoint, steps=30):
		points = []
		delta = (outputPoint[0]-inputPoint[0], outputPoint[1]-inputPoint[1])
		for i in xrange(steps+1):
			theta = float(i)/steps
			x = inputPoint[0] + delta[0] * theta 
			y = inputPoint[1] + delta[1] * (1-math.cos(theta*math.pi))/2
			points.append( (x,y) )
		return points

	# END class SimpleEdgePolicy
	pass



class SimpleVerticalEdgePolicy(EdgePolicy):

	def __init__(self):
		self.lineColour([0.75, 0.75, 0.0, 1.0])
		return
		
	"""
	def setupEdge(self, edge):
		edge.children = []
		if edge.inputNode is None or edge.outputNode is None:
			return
					
		try:
			inputPort = edge.inputPort()
			outputPort = edge.outputPort()
			
			point1 = (edge.inputNode.x + inputPort.x + inputPort.width/2, edge.inputNode.y + inputPort.y + inputPort.height/2)
			point2 = (edge.outputNode.x + outputPort.x + outputPort.width/2, edge.outputNode.y + outputPort.y + outputPort.height/2)
		except KeyError:
			return
		
		line = PrimitivesModule.zglLine()
		line.colour = self._lineColour
		line.points = self.createPath(point1, point2, 30)
		edge.children.append(line)
		
		self.updateEdge(edge)
		return
	"""

	
	def updateEdge(self, edge):
		
		inputPort = edge.inputPort()
		inputPort.edge = edge
		inputPort.updateColours()
		
		outputPort = edge.outputPort()
		outputPort.edge = edge
		outputPort.updateColours()
		
		pass

	
	def createPath(self, inputPoint, outputPoint, steps=30):
		points = []
		delta = (outputPoint[0]-inputPoint[0], outputPoint[1]-inputPoint[1])
		for i in xrange(steps+1):
			theta = float(i)/steps
			x = inputPoint[0] + delta[0] * (1-math.cos(theta*math.pi))/2
			y = inputPoint[1] + delta[1] * theta
			points.append( (x,y) )
		return points
		
	# END class SimpleVerticalEdgePolicy
	pass



#!/usr/bin/python

import logging
import math, sys, os, time

APP_ROOT = os.getenv('ZGL_HOME')
sys.path.insert(0, '%s/src' % APP_ROOT)


import zgl_graphdrawer.Application as ApplicationModule
import zgl_graphdrawer.GraphDrawerManager as ContextManagerModule
import zgl_graphdrawer.utils as CanvasUtils

import MockData as DataModule


	
if __name__ == "__main__":
	# setup logging
	CanvasUtils.configLogging()

	# execution stuff
	(mockNodes, mockEdges) = DataModule.createMockData()
	
	app = ApplicationModule.zglApplication()
	app.setDefaultResourcePath()
	# app.initializeContextManager()
	# contextManager = app.contextManager()
	contextManager = ContextManagerModule.GraphDrawerManager()
	app.contextManager(contextManager)
	contextManager.app(app)
	contextManager.initialize()

	# TODO:
	# this needs to be specified elsewhere
	contextManager.commandPath('dot', '/sw/bin/dot')

	app.initializePolicies()

	# some of us have different paths for dot
	if len(sys.argv) > 1:
		import getopt
		try:
			options, args = getopt.getopt(sys.argv[1:], "d:", ["dot="])
			for option, argument in options:
				if option in ("-d","--dot"):
					app.graphDrawerManager.commandPath('dot', argument)
		finally:
			pass
	
	contextManager.setDataContext(mockNodes, mockEdges)

	canvas = contextManager.canvas
	canvas.computeLayout()


	app.run()



import os, string, Image
import math, operator		# For vector stuff

from OpenGL.GL import *
from OpenGL.GLUT import *
from zglUtils import zglVector


class zglDrawer:
	@staticmethod
	def drawRectangle(x, y, width, height, colour):
		glPushMatrix()
		glTranslatef(x, y, 0)
		glColor4f(colour[0], colour[1], colour[2], colour[3])
		glBegin(GL_QUADS)
		glVertex2f(0, height)
		glVertex2f(width, height)
		glVertex2f(width, 0)
		glVertex2f(0, 0)
		glEnd()
		glPopMatrix()

	@staticmethod
	def drawRectangleWithGradient(x, y, width, height, colour1, colour2, angle):
		glPushMatrix()
		glTranslatef(x, y, 0)
		glColor4f(colour[0], colour[1], colour[2], colour[3])
		glBegin(GL_QUADS)
		glVertex2f(0, height)
		glVertex2f(width, height)
		glVertex2f(width, 0)
		glVertex2f(0, 0)
		glEnd()
		glPopMatrix()

	# radius is a 4-tuple for topleft, topright, bottomleft, bottomright radius'
	@staticmethod
	def drawRoundedRectangle(x, y, width, height, radius, colour):
		glPushMatrix()
		glTranslatef(x, y, 0)
		glColor4f(colour[0], colour[1], colour[2], colour[3])
		glBegin(GL_QUADS)
		glVertex2f(0, height)
		glVertex2f(width, height)
		glVertex2f(width, 0)
		glVertex2f(0, 0)
		glEnd()
		glPopMatrix()
		
	
	@staticmethod
	def drawLine(points):
		l = len(points[0])
		if l == 2:
			zglDrawer.drawLine2(points)
		elif l > 2:
			zglDrawer.drawLine3(points)
	
	@staticmethod
	def drawLine2(points):
		glBegin(GL_LINE_STRIP)
		for point in points:
			glVertex2f(point[0], point[1])
		glEnd()
	
	@staticmethod
	def drawLine3(points):
		#point1 = zglVector.lengthen(point1, 3)
		#point2 = zglVector.lengthen(point2, 3)
		
		glBegin(GL_LINE_STRIP)
		for point in points:
			glVertex3f(point[0], point[1], point[2])
		glEnd()

	@staticmethod
	def drawCircularArc(center, startAngle, endAngle):
		
		glBegin(GL_LINE_STRIP)
		
		glEnd()

	@staticmethod
	def drawBezierArc(point1, point2, controlPoint):
		pass
	
	
	


import logging
import sys
import time

from zgl_graphdrawer.Edge import *
from zgl_graphdrawer.Node import *
from zgl_graphdrawer.MouseHandlers import *
import zgl_graphdrawer.Event as EventModule

from zgl.zglUtils import zglUtils


import OpenGL
from OpenGL.GL import *

import numpy



class Canvas(object):

    def __init__(self):
        self.scroll_position = [0.0, 0.0]

        # set up the zoom and pan matrices
        self.zoomMatrix = numpy.matrix('1 0 0 0; 0 1 0 0; 0 0 1 0; 0 0 0 1')
        self.panMatrix = numpy.matrix('0 0 0 0; 0 0 0 0; 0 0 0 0; 0 0 0 0')

        self.selection_rectangle = None

        self.resetDrawables()
        return

    def contextManager(self, value=None):
        if value is not None:
            self._contextManager = value
        return self._contextManager


    def nodePolicy(self, value=None):
        if value is not None:
            self._nodePolicy = value
        return self._nodePolicy

    def edgePolicy(self, value=None):
        if value is not None:
            self._edgePolicy = value
        return self._edgePolicy

    def layoutPolicy(self, value=None):
        if value is not None:
            self._layoutPolicy = value
        return self._layoutPolicy

    def visualPolicy(self, value=None):
        if value is not None:
            self._visualPolicy = value
        return self._visualPolicy

    def portPolicy(self, value=None):
        if value is not None:
            self._portPolicy = value
        return self._portPolicy

    def resetDrawables(self):
        self._dynamicEdge = None
        self.nodes = []
        self.edges = []
        self.selectables = set([])
        self.clickables = set([])
        self.layers = {}
        for layerName in ['edges', 'nodes']:
            layer = CanvasLayer(layerName)
            self.layers[layerName] = layer
            pass
        return


    def addDrawable(self, drawable):
        if isinstance(drawable, Node):
            self.nodes.append(drawable)
            self.layers['nodes'].addChild(drawable)

            ports = drawable.inputPorts.values() + drawable.outputPorts.values()
            for port in ports:
                self.determineSelectable(port)
                self.determineClickable(port)

        if isinstance(drawable, Edge):
            self.edges.append(drawable)
            self.layers['edges'].addChild(drawable)

        self.determineSelectable(drawable)
        self.determineClickable(drawable)
        return


    def determineSelectable(self, object):
        if object.isSelectable():
            self.selectables.add(object)
        return

    def determineClickable(self, object):
        if object.isClickable():
            self.clickables.add(object)

        return

    def removeDrawable(self, drawable):
        return self.removeChild(drawable)
    
    def removeChild(self, drawable):
        # remove from layers
        for layer in self.layers.values():
            layer.removeChild(drawable)

        if isinstance(drawable, Node):
            ports = drawable.inputPorts.values() + drawable.outputPorts.values()
            for port in ports:
                self.selectables.discard(port)
                self.clickables.discard(port)

        self.selectables.discard(drawable)
        self.clickables.discard(drawable)
        return

    def getSelectables(self):
        return self.selectables

    def getClickables(self):
        return self.clickables

    def computeLayout(self):

        if not len(self.nodes):
            # no need to compute layout if self has no nodes
            return

        (x_max, y_max) = self.layoutPolicy().layoutNodes(
            self.nodes, self.edges)
        self.scroll_bounds = (x_max, y_max)

        self.updateLayout()
        return


    def updateLayout(self):

        map(lambda x: x.setupPrimitives(self.nodePolicy()), self.nodes)
        map(lambda x: x.setupPrimitives(self.edgePolicy()), self.edges)

        return


    def setScrollPosition(self, new_scroll_position):
        self.scroll_position = new_scroll_position
        self.validateScrollPosition()


    def validateScrollPosition(self):
        # this is really only used by zglCanvas
        # need to refactor to reflect that
        pass


    def computeDynamicEdge(self, initialPosition, finalPosition):

        line = PrimitivesModule.zglLine()
        edgePolicy = self.edgePolicy()

        initialCanvasMatrix = numpy.matrix('1 0 0 0; 0 1 0 0; 0 0 1 0; %s %s 0 1' % (initialPosition.x, initialPosition.y))
        finalCanvasMatrix = numpy.matrix('1 0 0 0; 0 1 0 0; 0 0 1 0; %s %s 0 1' % (finalPosition.x, finalPosition.y))

        initialWorldMatrix = self.getWorldCoordinatesFromCanvasCoordinates(initialCanvasMatrix)
        finalWorldMatrix = self.getWorldCoordinatesFromCanvasCoordinates(finalCanvasMatrix)

        point1 = (initialWorldMatrix[3,0], initialWorldMatrix[3,1])
        point2 = (finalWorldMatrix[3,0], finalWorldMatrix[3,1])

        line.points = edgePolicy.createPath(point1, point2)
        line.position = (0.0, 0.0, 0.01)
        line.colour = [0.7, 0.7, 1.0, 0.3]
        
        edgeClass = self.contextManager().app().getResourceValue(
            'gui edge class', default=Edge)
        self._dynamicEdge = edgeClass(None, None)
        self._dynamicEdge.children = [line]
        self.points = line.points

        pass


    def drawLayers(self):
        # Draw nodes
        for layerName, layer in self.layers.iteritems():
            for child in layer.getChildren():
                child.draw()
        return

    def drawDynamicEdge(self):
        if self._dynamicEdge is not None:
            self._dynamicEdge.draw()
        return

    def draw(self):
        glPushMatrix()

        glTranslate(self.panMatrix[3,0], self.panMatrix[3,1], self.panMatrix[3,2])
        glScale(1.0/self.zoomMatrix[0,0], 1.0/self.zoomMatrix[1,1], 1.0/self.zoomMatrix[2,2])

        self.drawLayers()

        self.drawDynamicEdge()

        # Draw selection rectangle
        if self.selection_rectangle is not None:
            zglUtils.drawRect(
                self.selection_rectangle[0], 
                self.selection_rectangle[1], 
                self.selection_rectangle[2], 
                self.selection_rectangle[3], (0.8, 0.5, 0.2, 0.2)
            )

        glPopMatrix()
        return


    def getObjectsIntersectingRect(self, rect):
        objects = []

        # recalculate the rect so that its in the form 
        # where width and height are both positive
        baseX = rect[0]
        baseY = rect[1]
        diffX = baseX + rect[2]
        diffY = baseY + rect[3]

        minX = min(baseX, diffX)
        minY = min(baseY, diffY)
        maxX = max(baseX, diffX)
        maxY = max(baseY, diffY)
        rect = (minX, minY, maxX-minX, maxY-minY)

        for child in self.nodes + self.edges:
            if child.intersectsRect(rect):
                objects.append(child)
        return objects


    # END class Canvas
    pass






class CanvasLayer(object):

    def __init__(self, name):
        self.name = name
        self._children = []
        return

    def addChild(self, child):
        self.getChildren().append(child)
        return

    def removeChild(self, child):
        while child in self.getChildren():
            self.getChildren().remove(child)
        return

    def getChildren(self):
        return self._children


    # END class CanvasLayer
    pass



import logging
import sys
import time

from zgl_graphdrawer.Edge import *
from zgl_graphdrawer.Node import *
from zgl_graphdrawer.MouseHandlers import *
import zgl_graphdrawer.Event as EventModule
import zgl_graphdrawer.canvas as CanvasModule
import zgl_graphdrawer.GraphDrawerManager as ContextManagerModule

from zgl.zglUtils import zglUtils


import OpenGL
from OpenGL.GL import *

import numpy



class Canvas(CanvasModule.Canvas):


    def __init__(self):
        CanvasModule.Canvas.__init__(self)
        self.frame = (0.0, 0.0, 0.0, 0.0)	# x_min, y_min, x_max, y_max of the layout
        self.scroll_bounds = (0.0, 0.0)

        self.resetDrawables()


        # self.currentMouseHandler = None
        self.mouseHandlers = {}
        self.defaultMouseHandler = None
        self.activeMouseMode = None

        self.is_dragging = False
        self.is_zooming = False

        return


    def activeMouseHandler(self):
        return self.mouseHandlers.get(
            self.activeMouseMode, 
            self.defaultMouseHandler)


    def draw(self):
        CanvasModule.Canvas.draw(self)
        return

    def updateFrame(self, frame):
        self.frame = frame
        self.validateScrollPosition()


    def children(self):
        for node in self.nodes:
            yield node
        for edge in self.edges:
            yield edge
        raise StopIteration



    def mouseMoved(self, x, y):
        self.activeMouseHandler().mouseMoved(x, y)


    def mouseButtonDown(self, button, x, y):
        # Check if any of the child has buttons that
        # consumed this event 
        for child in self.children():
            if child.mouseButtonDown(button, x, y):
                return True

        result = self.activeMouseHandler().mouseButtonDown(button, x, y)
        if result is not None and result != False:
            return result

        return

    def mouseDragged(self, x, y):

        self.activeMouseHandler().mouseDragged(x, y)

        return

    def mouseButtonUp(self, button, x, y):
        for child in self.children():
            if child.mouseButtonUp(button, x, y):
                return True

        self.activeMouseHandler().mouseButtonUp(button, x, y)
        return


    def initializeEventHandlers(self, contextManager=None):

        self.activeMouseMode = ContextManagerModule.GraphDrawerManager.MOUSEMODE_SELECT

        # Mouse Handlers
        scrollMouseHandler = CanvasScrollMouseHandler(self)
        zoomMouseHandler = CanvasZoomMouseHandler(self)
        selectionMouseHandler = CanvasSelectionMouseHandler(
            self, contextManager.selection)

        mouseHandlers = {
            ContextManagerModule.GraphDrawerManager.MOUSEMODE_SELECT:selectionMouseHandler,
            ContextManagerModule.GraphDrawerManager.MOUSEMODE_CANVAS_PAN:scrollMouseHandler,
            ContextManagerModule.GraphDrawerManager.MOUSEMODE_CANVAS_ZOOM:zoomMouseHandler
        }

        # Default Mouse Handler
        self.defaultMouseHandler = selectionMouseHandler
        self.mouseHandlers = mouseHandlers
        return


    def getObjectAtViewCoordinates(self, x, y):
        (canvas_x, canvas_y) = self.getCanvasCoordinatesFromViewCoordinates(x, y)
        for child in self.children():
            if child.isInside(x, y):
                return child

        return None


    def getViewCoordinatesFromCanvasCoordinates(self, x, y):
        view_x = x/self.zoom - self.scroll_position[0];
        view_y = y/self.zoom - self.scroll_position[1];
        return (view_x, view_y)

    def getCanvasCoordinatesFromViewCoordinates(self, x, y):
        canvas_x = int( (self.scroll_position[0] + x)*self.zoom )
        canvas_y = int( (self.scroll_position[1] + y)*self.zoom )
        return (canvas_x, canvas_y)

    def validateScrollPosition(self):
        if self.scroll_position[0] + self.frame[0] > self.scroll_bounds[0]:
            self.scroll_position[0] = self.scroll_bounds[0] - self.frame[0]
        elif self.scroll_position[1] > self.scroll_bounds[1]:
            pass


    '''
    def displayFunc(self):
            glDrawBuffer(GL_BACK)
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

            glDisable(GL_LIGHTING)
            glColor3f(1.0, 1.0, 1.0)

            glMatrixMode(GL_MODELVIEW)
            glLoadIdentity()

            """main drawing stuff goes here"""
            glPushMatrix()
            self.graphDrawerManager.step()
            self.graphDrawerManager.draw()
            glPopMatrix()

            x = 30
            y = 30
            glColor3f(1.0, 1.0, 1.0)
            zglUtils.zglUtils.drawBitmapText("Hello World", x, y, )
            self.drawFPS()

            glutSwapBuffers()

    def drawFPS(self):
            if hasattr(self, "last_t"):
                    self.fps = 1.0/float(time.time()-self.last_t)
                    glColor3f(1.0, 1.0, 1.0)
                    zglUtils.zglUtils.drawBitmapText("FPS: %f" % (self.fps), 30, self.height-30, font=GLUT_BITMAP_HELVETICA_10)
            self.last_t = time.time()

    '''

    # END class zglCanvas
    pass





class CanvasLayer(object):

    def __init__(self, name):
        self.name = name
        self._children = []
        return

    def addChild(self, child):
        self.getChildren().append(child)
        return

    def removeChild(self, child):
        while child in self.getChildren():
            self.getChildren().remove(child)
        return

    def getChildren(self):
        return self._children


    # END class CanvasLayer
    pass



#!/usr/bin/python

import logging
import math, sys, os, time


import zgl_graphdrawer.Application as ApplicationModule

def createMockData():
	mockNodes = []
	for i in range(8):
		nodeData = MockNodeData()
		nodeData.name("Job %d" % (i))
		if i < 4:
			nodeData.inputPorts = ['input1', 'input2', 'input3']
		else:
			nodeData.inputPorts = ['input1']
		nodeData.outputPorts = ['result']
		mockNodes.append(nodeData) 

	mockEdges = []
	mockEdge = MockEdgeData()
	mockEdge.inputNode = lambda: mockNodes[0]
	mockEdge.inputPort = 'result'
	mockEdge.outputNode = lambda: mockNodes[1]
	mockEdge.outputPort = 'input1'
	mockEdges.append(mockEdge)

	mockEdge = MockEdgeData()
	mockEdge.inputNode = lambda: mockNodes[0]
	mockEdge.inputPort = 'result'
	mockEdge.outputNode = lambda: mockNodes[2]
	mockEdge.outputPort = 'input2'
	mockEdges.append(mockEdge)

	mockEdge = MockEdgeData()
	mockEdge.inputNode = lambda: mockNodes[0]
	mockEdge.inputPort = 'result'
	mockEdge.outputNode = lambda: mockNodes[3]
	mockEdge.outputPort = 'input3'
	mockEdges.append(mockEdge)

	mockEdge = MockEdgeData()
	mockEdge.inputNode = lambda: mockNodes[0]
	mockEdge.inputPort = 'result'
	mockEdge.outputNode = lambda: mockNodes[4]
	mockEdge.outputPort = 'input1'
	mockEdges.append(mockEdge)

	for source, target in zip(mockNodes[4:-1], mockNodes[5:]):
		mockEdge = MockEdgeData()
		mockEdge.inputNode = lambda: source
		mockEdge.inputPort = 'result'
		mockEdge.outputNode = lambda: target
		mockEdge.outputPort = 'input1'
		mockEdges.append(mockEdge)

	return (mockNodes, mockEdges)

	
# Mock Data Model for node
class MockNodeData(object):
	def __init__(self):
		self.name("Node")

	def name(self, value=None):
		if value is not None:
			self._name = value
		return self._name
		
# Mock Data Model for edge
class MockEdgeData(object):
	def __init__(self):
		self.inputNode = None
		self.outputNode = None

	def name(self, value=None):
		if value is not None:
			self._name = value
		if not hasattr(self, '_name'):
			return "%s.%s->%s.%s" % (self.inputNode().name(),
						 self.inputPort,
						 self.outputNode().name(),
						 self.outputPort)
		return self._name



import copy
import math

import VisualPolicy as VisualPolicyModule

import zgl.zglPrimitives as PrimitivesModule
import zgl.zglText as TextModule
import zgl.zglUtils as zglUtils

import zgl_graphdrawer.Port as PortModule

class NodePolicy(object):

    KEY_BORDER_SELECTION_TRUE = "selected border"
    KEY_BORDER_SELECTION_FALSE = "not selected border"
    KEY_BACKGROUND_SELECTION_TRUE = "selected background"
    KEY_BACKGROUND_SELECTION_FALSE = "not selected background"
    KEY_TEXT_SELECTION_TRUE = "selected text"
    KEY_TEXT_SELECTION_FALSE = "not selected text"

    @staticmethod
    def constructDefaultColours(nodePolicy):
        
        import Node as NodeModule
        
        visualPolicy = nodePolicy.visualPolicy()

        colourBorderSelectionTrue = visualPolicy.colour(
            "%s, %s" % (
                NodeModule.Node.PART_BORDER,
                VisualPolicyModule.VisualPolicy.KEY_SELECTION_TRUE
            )
        )
        colourBorderSelectionFalse = visualPolicy.colour(
            "%s, %s" % (
                NodeModule.Node.PART_BORDER,
                VisualPolicyModule.VisualPolicy.KEY_SELECTION_FALSE
            )
        )
        colourBackgroundSelectionTrue = visualPolicy.colour(
            "%s, %s" % (
                NodeModule.Node.PART_BACKGROUND,
                VisualPolicyModule.VisualPolicy.KEY_SELECTION_TRUE
            )
        )
        colourBackgroundSelectionFalse = visualPolicy.colour(
            "%s, %s" % (
                NodeModule.Node.PART_BACKGROUND,
                VisualPolicyModule.VisualPolicy.KEY_SELECTION_FALSE
            )
        )
        colourTextSelectionTrue = visualPolicy.colour(
            "%s, %s" % (
                NodeModule.Node.PART_LABEL,
                VisualPolicyModule.VisualPolicy.KEY_SELECTION_TRUE
            )
        )
        colourTextSelectionFalse = visualPolicy.colour(
            "%s, %s" % (
                NodeModule.Node.PART_LABEL,
                VisualPolicyModule.VisualPolicy.KEY_SELECTION_FALSE
            )
        )


        colours =  {
            NodePolicy.KEY_BORDER_SELECTION_TRUE:colourBorderSelectionTrue,
            NodePolicy.KEY_BORDER_SELECTION_FALSE:colourBorderSelectionFalse,
            NodePolicy.KEY_BACKGROUND_SELECTION_TRUE:colourBackgroundSelectionTrue,
            NodePolicy.KEY_BACKGROUND_SELECTION_FALSE:colourBackgroundSelectionFalse,
            NodePolicy.KEY_TEXT_SELECTION_TRUE:colourTextSelectionTrue,
            NodePolicy.KEY_TEXT_SELECTION_FALSE:colourTextSelectionFalse
        }

        return colours


    def __init__(self, contextManager):
        self.contextManager(contextManager)
        return

    def contextManager(self, value=None):
        if value is not None:
            self._manager = value
        return self._manager

    def initializeColours(self):
        self.resetColours()
        return
    
    
    def resetColours(self):
        self._colours = NodePolicy.constructDefaultColours(self)
        return

    def colour(self, key, value=None):
        if value is not None:
            self._colours[key] = value
        return self._colours[key]


    def visualPolicy(self, value=None):
        if value is not None:
            self._visualPolicy = value
        return self._visualPolicy

    def portPolicy(self, value=None):
        if value is not None:
            self._portPolicy = value
        return self._portPolicy

    
    def getHorizontalAlignmentOfLabel(self):
        return TextModule.ALIGN_CENTER
        
    def getVerticalAlignmentOfLabel(self):
        return TextModule.ALIGN_MIDDLE
    
    def getPositionForNameLabel(self):
        return [0, 0, 0]


    def setupDimensions(self, node):
        node.width=100
        node.height=50
        return

    
    # END class NodePolicy
    pass


class SimpleNodePolicy(NodePolicy):

    def __init__(self, contextManager):
        NodePolicy.__init__(self, contextManager)
        return



    def boundsCheck(self, node_x, node_y, node_width, node_height, x, y):
        if x < node_x or x > node_width:
            return False
        if y < node_y or y > node_height:
            return False
        return True

    
    def updateNode(self, node):

        node.shouldUpdateDisplay(True)

        return

    def setPortPosition(self, node, port):

        # TODO:
        # some of the values in this function
        # are calculated multiple times
        # because this function is called once per port
        # but some of the values, 
        # e.g. minimumPortAreaWidth
        # is constant


        portDirection = port.direction

        yOffset = 0
        if (portDirection == PortModule.Port.PORT_DIRECTION_INPUT):
            yOffset = node.height + 5
        elif (portDirection == PortModule.Port.PORT_DIRECTION_OUTPUT):
            yOffset = -1*(port.height+5)

        port.y = yOffset

        minimumPortMargin = 5
        portCount = port.positionCount()
        minimumPortAreaWidth = \
            port.width * portCount + minimumPortMargin * (portCount-1)

        portIndex = port.positionIndex()

        xOffset = 0
        if minimumPortAreaWidth > node.width:
            # Ports have to extend beyond the limit of the node
            # because of size constraint
            startLocation = (node.width-minimumPortAreaWidth)/2
            xOffset = startLocation + portIndex*(portWidth + minimumPortMargin) - portWidth/2
        else:
            xOffset = node.width/portCount * (portIndex+0.5) - port.width/2
        port.x = xOffset


        return 

    # END class SimpleNodePolicy
    pass



import logging

from Responder import *

import zgl.zglPrimitives as PrimitivesModule

import zgl_graphdrawer.Menu as MenuModule


class Port(Responder):

    PART_BACKGROUND = "port background"

    PORT_DIRECTION_INPUT = 1
    PORT_DIRECTION_OUTPUT = 2

    @staticmethod
    def hasCustomDragReleaseCallback():
        return True


    @staticmethod
    def OnDrag(event, canvas=None,
               objects=None, 
               basePosition=None,
               resetCallback=None,
               shouldResetCallbackFunction=None):

        if shouldResetCallbackFunction(event):
            resetCallback()
            pass


        canvas.computeDynamicEdge(basePosition, event.GetPosition())

        pass


    def __init__(self, portName):
        Responder.__init__(self)
        self.x = 0
        self.y = 0
        self.width = 0
        self.height = 0
        self.children = []
        self.name = portName
        self.edge = None
        self.basePosition = (0, 0)
        return

    def OnDragRelease(self, event, canvas=None, 
                      objects=None,
                      eventHandler=None):

        if canvas is None:
            raise ValueError('canvas should not be None')

        # TODO:
        # should check if released on a port
        # that can be connected to the initial port
        # and if so, create an actual edge
        releasePosition = event.GetPosition()

        contextManager = canvas.contextManager()
        if contextManager is None:
            raise ValueError('canvas should have access to the context manager')

        clickReleaseObject = canvas.getClickableObjectAtCanvasCoordinates(
            releasePosition.x, releasePosition.y)

        edge = None
        if clickReleaseObject is not None and \
           contextManager.canConnect(self, clickReleaseObject):
            edge = contextManager.connect(self, clickReleaseObject)
            pass

        # reset the dynamic edge
        # (because there won't be one anymore)
        canvas._dynamicEdge = None

        # reset the mouse moved callback
        eventHandler.resetMouseMovedCallback()

        return edge

    
    def initialSetup(self):
        """
        This is a placeholder
        that subclasses can override and 
        hook in custom configuration
        """
        return
    
    def positionIndex(self, value=None):
        if value is not None:
            self._positionIndex = value
        return self._positionIndex

    def positionCount(self, value=None):
        if value is not None:
            self._positionCount = value
        return self._positionCount


    def isClickable(self):
        return True

    def updateColours(self):

        # No color policy for Ports set up yet 
        #colours = self.parts["colours"]

        background = self.parts[Port.PART_BACKGROUND]
        if self.edge is not None:
            background.borderColour = [0.8, 0.8, 0.0, 1.0]
            # background.colour = [0.5, 0.5, 0.0, 1.0]
        else:
            background.borderColour = [0.8, 0.8, 0.0, 1.0]
            background.colour = None
            pass

        return


    def overlaps(self, point):
        """
        TODO:
        This is copied from Node.Node
        Need to remove duplication
        """

        selfX = self.x + self.node.x
        selfY = self.y + self.node.y

        if point.x < selfX:
            return False
        if point.x > selfX+self.width:
            return False
        if point.y < selfY:
            return False
        if point.y > selfY+self.height:
            return False
        return True


    def draw(self):
        glPushMatrix()
        glTranslate(self.x, self.y, 0)

        for child in self.children:
            child.draw()

        glPopMatrix()

        return

    def getSelectionContextualMenu(self, event, 
                                   eventHandler, canvas, selection):
        popupMenu = SelectionContextualMenu(
            event, eventHandler, canvas, selection)
        return popupMenu


    def createBackgroundPrimitive(self, portPolicy):
        background = PrimitivesModule.zglRect()
        background.position = [0, 0, 0]
        background.size = [self.width, self.height, 1.0]
        background.corner_mode = True
        background.colour = None
        background.borderColour = [0.5, 0.5, 0.0, 1.0]
        return background

    
    def setupPrimitives(self, portPolicy):
        
        background = self.createBackgroundPrimitive(portPolicy)
        
        self.parts = {}
        self.parts[Port.PART_BACKGROUND] = background

        self.children.append(background)

        return

    # END class Port
    pass


class SelectionContextualMenu(MenuModule.SelectionContextualMenu):

    def bindEvents(self):
        pass
    pass

